from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ItemRevision
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ReassignParticipantInfo(TcBaseObj):
    """
    A structure containing  a list of input objects  whose participant roles are reassigned, assignee to reassign from,
    assignee to reassign to, a list of participant types, a boolean to indicate all participant types and comments.
    
    :var itemRevs: List of ItemRevision objects to reassign participants roles.
    :var fromAssignee: The business object (User, GroupMember or a ResourcePool) to be replaced with the toAssignee
    business object.
    :var toAssignee: The business object (User, GroupMember or a ResourcePool) to replace the fromAssignee business
    object.
    :var participantTypes: List of participant types to be reassigned.  Valid types are:
    - ProposedResponsibleParty
    - ProposedReviewer
    - ChangeImplementationBoard
    - ChangeReviewBoard
    - ChangeSpecialist1
    - ChangeSpecialist2
    - ChangeSpecialist3
    - Analyst
    - Requestor
    
    
    Custom participant types are also allowed.
    :var allParticipantTypes: If true, all the participant roles for the fromAssignee will be considered for
    reassignment.  The values provided in the participantTypes list will be ignored.
    
    If false, only the participant roles provided in the particpantTypes list will be considered.
    :var comments: A comment provided during reassignment.
    :var clientId: A unique string supplied by the caller. This ID is used to identify the return data elements and
    partial errors associated with the input structure.
    """
    itemRevs: List[ItemRevision] = ()
    fromAssignee: BusinessObject = None
    toAssignee: BusinessObject = None
    participantTypes: List[str] = ()
    allParticipantTypes: bool = False
    comments: str = ''
    clientId: str = ''


@dataclass
class ReassignParticipantOutput(TcBaseObj):
    """
    Contains clientId and a list of objects on which the reassign operation failed.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify the return data elements and
    partial errors associated with the input structure.
    :var failedItemRevs: A list of ItemRevision objects on which the reassign operation failed.
    """
    clientId: str = ''
    failedItemRevs: List[ItemRevision] = ()


@dataclass
class ReassignParticipantResponse(TcBaseObj):
    """
    Contains a list of failed objects information and ServiceData.
    
    :var failedObjects: A list of failed objects information.
    :var serviceData: The modified participant objects are returned along with partial errors.
    """
    failedObjects: List[ReassignParticipantOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, AppInterface
from tcsoa.gen.Ai._2009_06.Ai import GenerateScopedMultipleStructure3Response
from typing import List
from tcsoa.gen.Ai._2008_06.Ai import ObjectsWithConfig
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class AiService(TcService):

    @classmethod
    def generateScopedMultipleStructure3(cls, aiObject: AppInterface, objectsToProcess: List[ObjectsWithConfig], exportTransferMode: str, serverMode: int) -> GenerateScopedMultipleStructure3Response:
        """
        GenerateScopedMultipleStructure3: Same as GenerateScopedMultipleStructure2 - except filetickets are returned.
        If aiObject is specified - it is only
        used to get the TransferMode (in case the exportTransferMode is not specified).
        objects or occurrence group objects - specified as application refs. The configuration is optional
        if the ids consist of StructureContexts. ApplicationRefs can be ids of occurrence from a previous
        export from TC, or APNs or AbsOccs, or OccurrenceGroup or an Item/Itemrev(only one in that last case).
        If the Appref is custom (non TcEng AppRef), occurrence appref must resolve to AbsOccurrence or APN ),
        or they can be ids of structure context/occurrence group objects. The return is the transient file ticket
        for the plmxml file generated. In case any of the input apprefs cannot be processed they will be returned
        in the data member of response. Errors during plmxml processing will be returned in partialerror as
        xml string, based on TcError.xsd in iman_data folder. Configuration structure can be used to specify
        default revrule (if true) all other fields are ignored. Basically, they are declared in the order of
        precedence (where duplication is possible).
        """
        return cls.execute_soa_method(
            method_name='generateScopedMultipleStructure3',
            library='Ai',
            service_date='2009_06',
            service_name='Ai',
            params={'aiObject': aiObject, 'objectsToProcess': objectsToProcess, 'exportTransferMode': exportTransferMode, 'serverMode': serverMode},
            response_cls=GenerateScopedMultipleStructure3Response,
        )

    @classmethod
    def getPersistentObjects(cls, inputLines: List[BusinessObject]) -> ServiceData:
        """
        Given a set of bomlines from the same window, create a private structure context and return that. If the input
        contains any persistent objects like a workspaceobject - those will be returned as is.
        """
        return cls.execute_soa_method(
            method_name='getPersistentObjects',
            library='Ai',
            service_date='2009_06',
            service_name='Ai',
            params={'inputLines': inputLines},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.OccMgmt._2020_12.EffectivityManagement import GetEffectivityResponse, EffectivityInfoInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class EffectivityManagementService(TcService):

    @classmethod
    def setEffectivity(cls, effectivityDataInput: EffectivityInfoInput) -> ServiceData:
        """
        This operation sets effectivity on ItemRevision or Awb0Element objects. When input is ItemRevision the
        effectivity is set on ItemRevision. For Awb0Element, the effectivity is set on PSOccurrence associated with
        Awb0Element.
        End Item can be specified while setting the date and/or unit effectivity. If unit ranges are specified,
        effectivity end Item must be provided. A list of effectivity options can also be specified optionally along
        with date and unit ranges.
        
        Use cases:
        Use case 1: User adding effectivity on the object having no existing effectivity. 
        
        Input:
        Target Object existing effectivity: None 
        New Effectivity: Unit In = 3 &amp; Unit Out = 9 &amp; Intent = Design
        Resultant Effectivity set on the object: Unit In = 3 &amp; Unit Out = 9 &amp; Intent = Design
        
        Use case 2: User adding effectivity on the object having existing effectivity. 
        
        Input:
        Target Object existing effectivity: Unit In = 3 &amp; Unit Out = 9 &amp; Intent = Design
        New Effectivity: Unit In = 15 &amp; Unit Out = 20 &amp; Intent = Design
        Resultant Effectivity set on the object: ( Unit In = 3 &amp; Unit Out = 9 &amp; Intent = Design ) OR ( Unit In
        = 15 &amp; Unit Out = 20 &amp; Intent = Design )
        
        Use case 3: User adding effectivity on the object having existing effectivity. The new effectivity is
        overlapping with existing effectivity values.
        
        Input:
        Target Object existing effectivity: Unit In = 3 &amp; Unit Out = 9 &amp; Intent = Design
        New Effectivity: Unit In = 5 &amp; Unit Out = 20 &amp; Intent = Design
        Resultant Effectivity set on the object: Unit In = 5 &amp; Unit Out = 20 &amp; Intent = Design
        """
        return cls.execute_soa_method(
            method_name='setEffectivity',
            library='Internal-OccMgmt',
            service_date='2020_12',
            service_name='EffectivityManagement',
            params={'effectivityDataInput': effectivityDataInput},
            response_cls=ServiceData,
        )

    @classmethod
    def getEffectivity(cls, inputObjects: List[BusinessObject]) -> GetEffectivityResponse:
        """
        This operation retrieves the effectivity for the BusinessObjects of type ItemRevision or Awb0Element.
        """
        return cls.execute_soa_method(
            method_name='getEffectivity',
            library='Internal-OccMgmt',
            service_date='2020_12',
            service_name='EffectivityManagement',
            params={'inputObjects': inputObjects},
            response_cls=GetEffectivityResponse,
        )



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def createAppUidObjects(cls, count: int) -> ServiceData:
        """
        Create appuid object for datasets
        An appuid is an NX concept which maps to an instance of a POM application object. This service will create a
        set of empty appuid objects for the caller.
        
        
        Use cases:
        NX stores their CAD data in Teamcenter with dataset instances of type UGMASTER. NX only allows one data file
        (i.e. .prt) to be stored with a dataset. There is also a limit of one UGMASTER dataset per item revision. When
        NX needs to associate multiple data files (such as drawings, cam files etc ) with an item revision, they need a
        mechanism to be able to find the data file and know which dataset it is stored with. To do this NX creates an
        appuid and associates it with the dataset using the named reference name APPUID. They also store the uid of the
        appuid in the data file. When they need to access the data file they extract the appuid uid from the data file
        then search for the dataset that contains this appuid using the 'GetItemAndRelatedObjects' service. They can
        then either download the data file or update it. 
        This operation allows NX to create a series of POM application objects at one time to associate with the data
        files they are about to check in.
        """
        return cls.execute_soa_method(
            method_name='createAppUidObjects',
            library='Internal-Cad',
            service_date='2007_12',
            service_name='DataManagement',
            params={'count': count},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class IPAManagementGenerateSearchScopeResponse(TcBaseObj):
    """
    Return structure for IPAManagementGenerateSearchScope operation
    
    :var bomlines: A vectoer of all the bomlines that are the search scope.
    :var serviceData: This is a common data strucuture used to return sets of Teamcenter
     Data Model object from a service request. This also holds services exceptions.
    """
    bomlines: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class IPAManagementGetFilteredIPAResponse(TcBaseObj):
    """
    Return structure for IPAManagementGetFilteredIPA operation
    
    :var filteredIPAs: A vectoer of all the filteredIPAs found.
    :var serviceData: This is a common data strucuture used to return sets of Teamcenter
     Data Model object from a service request. This also holds services exceptions.
    """
    filteredIPAs: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class IPAManagementSaveSearchResultInput(TcBaseObj):
    """
    Input structure for IPAManagementSaveSearchResult operation
    
    :var process: The process from which we want to filter the IPA.
    :var searchResultList: The bomlines result from the search.
    :var name: The name of the new filtered structure.
    """
    process: BusinessObject = None
    searchResultList: List[BusinessObject] = ()
    name: str = ''


@dataclass
class IPAManagementSaveSearchResultResponse(TcBaseObj):
    """
    Return structure for IPAManagementSaveSearchResult operation
    
    :var filteredIPA: This is the new filteredIPA structure.
    :var filteredIPARoot: The flat list of all filteredIPA structures.
    :var rejectedList: A list of all the bomlines that were not found in the IPA structure.
    :var serviceData: This is a common data strucuture used to return sets of Teamcenter
     Data Model object from a service request. This also holds services exceptions.
    """
    filteredIPA: BusinessObject = None
    filteredIPARoot: BusinessObject = None
    rejectedList: List[BusinessObject] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Classification._2009_10.Classification import KeyLOVDefinition2
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetDependencyKeyLOVsResponse(TcBaseObj):
    """
    Holds the values returned by getKeyLOVsForDependentAttributes operation.
    
    :var dependencyKeyLOVs: A list containing attribute ID, key of to be selected entry from a KeyLOV and its KeyLOV
    definition for each to be changed attribute.
    :var serviceData: Any failures will be returned in the service data list of partial errors.
    """
    dependencyKeyLOVs: List[DependencyKeyLOVDescriptor] = ()
    serviceData: ServiceData = None


@dataclass
class DependencyAttributeStruct(TcBaseObj):
    """
    The structure containing class ID, the changed attribute ID, its value and all other UI attribute IDs and
    corresponding values.
    
    :var classID: The unique ID of Classification class.
    :var selectedAttributeID: The unique ID of an attribute (from above class) whose UI value is changed.
    :var selectedValue: The user selected value of the attribute (key of the selected entry from the KeyLOV). This
    could be empty if the value has been deselected.
    :var attributeValues: A list of DependencyAttributeValue objects containing attribute IDs and their corresponding
    values for all other UI attributes from above class.
    """
    classID: str = ''
    selectedAttributeID: int = 0
    selectedValue: str = ''
    attributeValues: List[DependencyAttributeValue] = ()


@dataclass
class DependencyAttributeValue(TcBaseObj):
    """
    Structure containing the attribute ID and corresponding value. 
    
    :var attributeID: Unique ID of an attribute.
    :var value: The UI value for above attribute (the key of a KeyLOV entry in case of KeyLOV attribute).
    """
    attributeID: int = 0
    value: str = ''


@dataclass
class DependencyKeyLOVDescriptor(TcBaseObj):
    """
    Structure representing dependent attribute IDs and corresponding key-value pairs. 
    
    :var attributeID: The unique ID of the changed attribute.
    :var selectedKeys: A list of unique key of to be selected entry for this attribute (can be empty).
    :var keyLOVDefinition: The structure representing the configured KeyLOV definition for attribute.
    """
    attributeID: int = 0
    selectedKeys: List[str] = ()
    keyLOVDefinition: KeyLOVDefinition2 = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, EPMTask
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateSignoffInfo(TcBaseObj):
    """
    'CreateSignoffInfo' structure contains the requisite information to create the Signoff object. If the 'originType
    'is "SOA_EPM_SIGNOFF_ORIGIN_PROFILE" and the signoff member already exit in
    any signoff member matches the signoff profile.  
    
    :var signoffMember: Signoff member is the reviewer who needs to perform the signoff action. It can be group member,
    resource pool or an address list. 
    :var origin: Origin references the source of the signoff. When a profile signoff is added, origin references the
    signoff profile object.When a member of an address list is added as signoff, origin represents the address list to
    which the signoff member belongs.  
    :var signoffAction: Possible values are 
    "SOA_EPM_Review"
     -This values specifies that the signoff needs to be added as a review signoff.
    
    "SOA_EPM_Acknowledge"
     -This value specifies that the signoff needs to be addaed as a acknowledge signoff.
     
    "SOA_EPM_Notify" 
     -This value specifies that the signoff needs to be addaed as a notify signoff.
    
    "SOA_EPM_ACTION_UNDEFINED"
     -This value should not be used in parameter, else an error would be returned.  
    
    :var originType: For profile signoff use case , the 'originType' is "SOA_EPM_SIGNOFF_ORIGIN_PROFILE" and for subset
    of addresslist use case , the 'originType' is "SOA_EPM_SIGNOFF_ORIGIN_ADDRESSLIST". 
    
    Possible string values:
    "SOA_EPM_ORIGIN_UNDEFINED"
      -This value specifies that the origin value is NULL or not defined.
    
    "SOA_EPM_SIGNOFF_ORIGIN_PROFILE"
      -This value is specifies that the origin value is the signoff profile object.
    
    "SOA_EPM_SIGNOFF_ORIGIN_ADDRESSLIST"
      -This value is specifies that the origin value is an address list object
              
    :var signoffRequired: string which indicates whether the added reviewer decision is required or optional while
    performing signoff.
    
    
    Possible values are
    "SOA_EPM_SIGNOFF_OPTIONAL"
    -sets the property to Optional. sign off decision is not required as long as quorum is met and can be manualy
    overridden to "Required Modifiable" before perform signoff task starts.
    
    "SOA_EPM_SIGNOFF_REQUIRED_MODIFIABLE"
    - sets the property to "Required Modifiable". sign off decision is required even if quorum is met and can be
    manually overriden back to "Optional" before perform signoff task starts.
    
    "SOA_EPM_SIGNOFF_REQUIRED_UNMODIFIABLE"
    - sets the property to "Required Unmodifiable"  signoff decision is required even if quorum is met and cannot be
    manually overridden to "Optional" before perform signoff task starts.
    
    
    
    
     
    """
    signoffMember: BusinessObject = None
    origin: BusinessObject = None
    signoffAction: str = ''
    originType: str = ''
    signoffRequired: str = ''


@dataclass
class CreateSignoffs(TcBaseObj):
    """
    'CreateSignoffs' structure contains the workflow task for which Signoff needs to be added and information to create
    the Signoff objects like the Signoff Member and references like Signoff Profile or address list. 
    
    :var task: Workflow task object to which the Signoff need to be added
    :var signoffInfo: The required information to create the Signoff object
    """
    task: EPMTask = None
    signoffInfo: List[CreateSignoffInfo] = ()



from __future__ import annotations

from tcsoa.gen.Internal.StructureManagement._2012_09.StructureVerification import ValidCriteriaInput, GetValidCriteriaResponse
from typing import List
from tcsoa.base import TcService


class StructureVerificationService(TcService):

    @classmethod
    def getValidCriteria(cls, inputscope: List[ValidCriteriaInput]) -> GetValidCriteriaResponse:
        """
        This operation is to be used when comparing two structures, and equivalence criteria is to be obtained. The
        input source scopes target scopes are passed to EquivalenceCriteria objects on the server - which determine if
        they are valid for this scenario or not. The result is a well defined set of keys that uniquely identify the
        EquivalenceCriteria object(s) that are valid.
        """
        return cls.execute_soa_method(
            method_name='getValidCriteria',
            library='Internal-StructureManagement',
            service_date='2012_09',
            service_name='StructureVerification',
            params={'inputscope': inputscope},
            response_cls=GetValidCriteriaResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ResolveDataInfo(TcBaseObj):
    """
    Input structure specifying the Operation or Process. the Tool Requirement that is to be resolved. the Tool with
    which Tool requirement resolve against and the tool source from where tool is to be picked.
    
    :var parent: Specifies the Operation of type Mfg0BvrOperation or Process of type Mfg0BvrProcess under which Tool
    requirement is assigned.
    :var trObject: Specifies the Tool Requirement of type Mfg0BVRToolRequirement that is to be resolved.
    :var toolObject: Tool of type Mfg0BvrResource that matches the search criteria of the Tool Requirement and against
    which it is going to be resolved.
    :var toolSource: Specifies the source from where the tool is to be fetched.
    The possible values are LIBRARY & WORKAREA and ALL which specify that the tool is to be fetched respectively from
    only library only workarea or workarea and library. Note that in the case of ALL the preference is given to the
    workarea over the library.
    """
    parent: BusinessObject = None
    trObject: BusinessObject = None
    toolObject: BusinessObject = None
    toolSource: str = ''


@dataclass
class Tool(TcBaseObj):
    """
    Specifies the candidate tools with members as the Tool Requirement and the tools that match the search criteria of
    Tool Requirement.
    
    :var trObject: Specifies the Tool Requirement of type Mfg0BVRToolRequirement.
    :var toolObjects: Specifies the candidate tools of type Mfg0BvrResource matching the search criteria of the Tool
    Requirement.
    """
    trObject: BusinessObject = None
    toolObjects: List[BusinessObject] = ()


@dataclass
class ToolRequirementInput(TcBaseObj):
    """
    Input structure specifying the Operation or Process. the Tool Requirement for which candidate tools are to be
    fetched and the tool source from where tools are to be fetched.
    
    :var parent: Specifies the Operation of type Mfg0BvrOperation or Process of type Mfg0BvrProcess under which Tool
    requirement is assigned.
    :var trObject: Specifies the Tool Requirement of type Mfg0BVRToolRequirement for which candidate tools are to be
    fetched.
    :var toolSource: Specifies the source from where the tools are to be fetched. 
    The possible values are Library & WorkArea and All. Which specify that the tools are to be fetched respectively
    from library and workarea. Only library and only workarea.
    """
    parent: BusinessObject = None
    trObject: BusinessObject = None
    toolSource: str = ''


@dataclass
class ToolRequirementResponse(TcBaseObj):
    """
    Specifies the Tool Requirements that are assigned to an Operation or Process.
    
    :var toolRequirements: Specifies the Tool Requirements.
    :var serviceData: Service data will hold warnings and errors. if any.
    """
    toolRequirements: List[ToolRequirementResult] = ()
    serviceData: ServiceData = None


@dataclass
class ToolRequirementResult(TcBaseObj):
    """
    Specifies the Tool Requirements that are assigned to an Operation or Process.
    
    :var parentObject: Specifies the Operation of type Mfg0BvrOperation or Process of type Mfg0BvrProcess to which Tool
    requirement is assigned.
    :var toolRequirements: Specifies the Tool Requirement of type Mfg0BVRToolRequirement that are assigned to the
    Operation or Process.
    """
    parentObject: BusinessObject = None
    toolRequirements: List[BusinessObject] = ()


@dataclass
class CandidateTool(TcBaseObj):
    """
    Specifies the candidate tools for the Operation or Process where Tool Requirement is assigned.
    
    :var parentObject: Specifies the Operation of type Mfg0BvrOperation or Process of type Mfg0BvrProcess where Tool
    Requirement is assigned.
    :var tools: Specifies the candidate tools matching the search criteria of the Tool Requirement.
    """
    parentObject: BusinessObject = None
    tools: List[Tool] = ()


@dataclass
class CandidateToolsForToolRequirement(TcBaseObj):
    """
    Specifies the candidate tools for the Tool Requirement
    
    :var candidateTools: Structure with member as Operation or Process and the candidate tools.
    :var serviceData: Service data will hold warnings and errors. if any.
    """
    candidateTools: List[CandidateTool] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetFilesAndTicketsInfoResponse(TcBaseObj):
    """
    The GetFilesAndTicketsInfoResponse holds the list of names and their FMS tickets for the first file as primary
    named reference associated with the Dataset and ServiceData.
    
    The response array will have a structure of file name and FMS Ticket that corresponds to every input Dataset object
    array in the same order. In case an element in the input array does not have an associated file name and ticket
    information , the response array will have a structure of blank string for that input and this input object will be
    passed in the partialError.
    
    :var serviceData: ServiceData for the operation.
    :var output: Response object containing a list of file name and its corresponding FMS ticket .
    """
    serviceData: ServiceData = None
    output: List[FileNameTicketOut] = ()


@dataclass
class FileNameTicketOut(TcBaseObj):
    """
    The FileNameTicketOut structure holds the file name and its corresponding FMS ticket for the first file of the
    primary named reference associated with the Dataset.
    
    :var fileName: The file name of the first file against the primary named reference associated with every Dataset
    :var fileTicket: FMS ticket of the first file against the primary named reference associated with every Dataset.
    """
    fileName: str = ''
    fileTicket: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from tcsoa.gen.Core._2011_06.OperationDescriptor import DeepCopyData
from typing import Dict, List


@dataclass
class GetDeepCopyDataResponse(TcBaseObj):
    """
    getDeepCopyData Response
    
    :var deepCopyInfoMap: Map of the DeepCopy data
    :var serviceData: Creates a list of Datasets and creates the specified relation type between created Dataset and
    input container object.
    """
    deepCopyInfoMap: DeepCopyInfoMap2 = None
    serviceData: ServiceData = None


@dataclass
class DeepCopyDataInput(TcBaseObj):
    """
    Input structure for getDeepCopyData operation
    
    :var operation: This is the operation types such as SaveAs,Revise, etc.
    :var object: object reference to get the deepcopydata
    """
    operation: str = ''
    object: BusinessObject = None


"""
Map contains a list of <name, value> pairs ('BusinessObject, vector<DeepCopyData>').  For each pair, name is the business object and value holds deep copy data for the business object.
"""
DeepCopyInfoMap2 = Dict[BusinessObject, List[DeepCopyData]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, TC_Project
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class AssignedOrRemovedObjects(TcBaseObj):
    """
    This structure holds the projects and workspace objects.
    
    :var projects: A list of TC_Project objects  to which the the given set of workspace objects need to be assigned or
    from which the objects need to be removed.
    :var objectToAssign: A list of objects that needs to be assigned ( added ) to the given projects.
    :var objectToRemove: A list of objects that needs to be removed  from the given projects.
    """
    projects: List[TC_Project] = ()
    objectToAssign: List[BusinessObject] = ()
    objectToRemove: List[BusinessObject] = ()



from __future__ import annotations

from typing import List
from tcsoa.gen.Ai._2012_09.Ai import RequestDetail
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindRequestOnAiWithReferencesResponse(TcBaseObj):
    """
    The latest RequestObjects (by creation date) that are on the AppInterface objects (latest by creation date) that
    reference the input baseRefs .
      The following partial errors may be returned:
    -     Invalid inputs
    -     Permission errors  
    
    
    
    :var details: Each element in the details (type _2012_09::Ai::RequestDetail) vector maps to the corresponding set
    based input vector and contains the following fields.
    :var serviceData: The standard soa serviceData that is used for adding partial errors and sending properties of the
    object. The properties that are returned by default ( without getting not loaded exception ) are:
    - object_name
    - object_desc
    
    """
    details: List[RequestDetail] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Cad._2007_01.DataManagement import ItemInfo, ExtraObjectInfo, ItemRevInfo, DatasetFileInfo, AttributeInfo
from tcsoa.gen.BusinessObjects import BusinessObject, Dataset
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class NamedReferenceObjectInfo2(TcBaseObj):
    """
    Contains information for object named references to apply to the Dataset.
    
    :var clientId: Identifier defined by user to track the related object.
    :var object: Object reference of the object for update, null for create
    :var namedReferenceName: The Named Reference from the dataset to this object, required. NamedReference values  are
    defined for each Dataset type. The customer can add more values as needed. To get a current list of valid Named
    Reference values the programmer can either use BMIDE or can call the SOA Core service getDatasetTypeIno.
    :var namedReferenceType: The reference type name from the dataset to this object, must be either AE_ASSOCIATION or
    AE_PART_OF.
    :var typeName: Type of the object to be created. Required for object creation only.
    :var attrNameValuePairs: List of AttributeInfos.
    """
    clientId: str = ''
    object: BusinessObject = None
    namedReferenceName: str = ''
    namedReferenceType: str = ''
    typeName: str = ''
    attrNameValuePairs: List[AttributeInfo] = ()


@dataclass
class PartInfo2(TcBaseObj):
    """
    The PartInfo2 struct is the main input to the createOrUpdateParts service.
    This structure refers to the Item, ItemRevision, and one or more Dataset structures used to create those objects.
    
    :var clientId: Identifier defined by user to track the related object.
    :var itemInput: Member of type ItemInfo
    :var itemRevInput: Member of type ItemRevInfo
    :var datasetInput: List of DatasetInfo2
    """
    clientId: str = ''
    itemInput: ItemInfo = None
    itemRevInput: ItemRevInfo = None
    datasetInput: List[DatasetInfo2] = ()


@dataclass
class CreateOrUpdatePartsPref(TcBaseObj):
    """
    Input structure for CreateOrUpdatePartsPref
    
    :var overwriteForLastModDate: Flag to check whether dataset needs to be modified, if input last modified date is
    different from actual.
    """
    overwriteForLastModDate: bool = False


@dataclass
class DatasetInfo2(TcBaseObj):
    """
    The DatasetInfo2 struct represents all of the data necessary to construct the dataset object.
    The basic attributes that are required are passed as named elements in the struct.
    All other attributes are passed as name/value pairs in the AttributeInfo struct.
    The extraObject field allows for the creation of an object(s) that will be related to this newly created Dataset.
    
    :var clientId: Identifier defined by user to track the related object.
    :var dataset: Dataset object reference for update, null for creation
    :var createNewVersion: Flag to create new version ( TRUE ) or not (FALSE )
    :var namedReferencePreference: Preference name which holds the list of named references to delete from one Dataset
    version to the next
    :var attrList: List of AttributeInfos for attributes
    :var mappingAttributes: List of AttributeInfos for mapped attributes. Mapped atributes are attributes that are
    applied to other objects. Refere to the ITK manual for a definition of attribute mapping.
    :var extraObject: List of ExtraObjectInfos
    :var datasetFileInfos: List of DatasetFileInfos
    :var namedReferenceObjectInfos: List of NamedReferenceObjectInfos
    :var name: Name attribute value
    :var basisName: Basis Name attribute value, used when the name is null or blank
    :var description: Description attribute value
    :var type: Type attribute value
    :var lastModifiedOfDataset: Last Modified Date of dataset
    :var id: ID attribute value
    :var datasetRev: Revision attribute value
    :var itemRevRelationName: Required input, may not be null, not defaulted
    """
    clientId: str = ''
    dataset: Dataset = None
    createNewVersion: bool = False
    namedReferencePreference: str = ''
    attrList: List[AttributeInfo] = ()
    mappingAttributes: List[AttributeInfo] = ()
    extraObject: List[ExtraObjectInfo] = ()
    datasetFileInfos: List[DatasetFileInfo] = ()
    namedReferenceObjectInfos: List[NamedReferenceObjectInfo2] = ()
    name: str = ''
    basisName: str = ''
    description: str = ''
    type: str = ''
    lastModifiedOfDataset: datetime = None
    id: str = ''
    datasetRev: str = ''
    itemRevRelationName: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ItemRevision
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class AlignmentScopeElement(TcBaseObj):
    """
    Contains the details of a single scope element for alignment.
    
    :var sourceScope: The BOMLine object representing the source scope to align with.
    :var targetScopes: The list of BOMLine objects representing the target scopes to be aligned.
    :var reusedNodes: The list of BOMLine objects under which the target BOMLine objects may have been assigned. All
    BOMLine objects assigned from source structure under this reusedNodes will be considered for alignment if
    alignmentMode is not specified. This list will be empty for single line alignment or  assemblies alignment. For
    reused assemblies alignment, the size of reusedNodes is the same as that of the targetScopes.
    """
    sourceScope: BusinessObject = None
    targetScopes: List[BusinessObject] = ()
    reusedNodes: List[BusinessObject] = ()


@dataclass
class AlignmentSummary(TcBaseObj):
    """
    Contains the summary information for a single alignment input.
    
    :var aligned: The number of successfully aligned lines.
    :var notAlignedWithError: The number of lines not aligned due to errors.
    :var notAlignedUnderReused: The number of lines under reused assemblies.
    """
    aligned: int = 0
    notAlignedWithError: int = 0
    notAlignedUnderReused: int = 0


@dataclass
class GetEquivalentPropValuesElement(TcBaseObj):
    """
    Input structure to provide property names, and original and new ItemRevisions to find equivalent property values
    for.
    
    :var propertyNamesByCategory: List of property names by category to find equivalent property values.
    :var originalItemRevision: The ItemRevision to get equivalent property values from.
    :var newItemRevision: The new ItemRevision for which the equivalent properties are needed.
    """
    propertyNamesByCategory: List[EquivalentPropertyNames] = ()
    originalItemRevision: ItemRevision = None
    newItemRevision: ItemRevision = None


@dataclass
class GetEquivalentPropValuesResp(TcBaseObj):
    """
    The equivalent properties and associated value and equivalent attachments.
    
    :var equivalentPropValuesElements: A list of getEquivalentPropValuesRespElem.
    The size of list is same as size of input to the operation.
    :var serviceData: Service data containing partial errors.
    """
    equivalentPropValuesElements: List[GetEquivalentPropValuesRespElem] = ()
    serviceData: ServiceData = None


@dataclass
class GetEquivalentPropValuesRespElem(TcBaseObj):
    """
    List of equivalent properties and values and equivalent attachments found.
    
    :var propertyCategory: Valid values are:
    - "Item"
    - "ItemRevision"
    - "Item Master"
    - "ItemRevision Master"
    
    
    :var equivalentPropertyValues: List of source and target equivalent properties and associated values.
    :var attachments: List of equivalent attachments found.
    """
    propertyCategory: str = ''
    equivalentPropertyValues: List[PropertyNamePairValue] = ()
    attachments: List[Attachment] = ()


@dataclass
class Attachment(TcBaseObj):
    """
    Description of attachment.
    
    :var relationName: Relation name between Item and attachment.
    :var action: Valid values are:
    - "Ignore" 
    -  "Reference" 
    -  "Clone"
    
    
    :var secondary: Attachment object associated to Item with relationName.
    """
    relationName: str = ''
    action: str = ''
    secondary: BusinessObject = None


@dataclass
class PasteOrReplaceAssemblyInContextInfo(TcBaseObj):
    """
    PasteOrReplaceAssemblyInContextInfo structure contains the input for paste /replace action on a given assembly
    (BOMLine). The structure will comprise of the new ItemRevision object ( needed in case of paste new / replace ),
    the  assembly (BOMLine)  to be pasted/replaced, the parent BOMLine object under which the paste is to be done and
    any additional info required. Currently this additional info is not used.
    
    :var sourceObject: The  BOMLine representing the root of assembly to be pasted/replaced.
    :var targetObject: The BOMLine representing the parent under which the sourceObject need to be pasted. If this is
    null then the sourceObject's item is replaced with newObject.
    :var newObject: The ItemRevision that will get a new BOMViewRevision, child lines, and in context edits duplicated
    from the sourceObject.  It will be pasted to targetObject.  If newObject is null then an ItemRevision of
    sourceObject will be pasted with in context edits duplicated.
    :var additionalInfo: The AdditionalInfo structure. Currently not used.
    """
    sourceObject: BusinessObject = None
    targetObject: BusinessObject = None
    newObject: BusinessObject = None
    additionalInfo: AdditionalInfo = None


@dataclass
class PasteOrReplaceAssemblyInContextResponse(TcBaseObj):
    """
    PasteOrReplaceAssemblyInContextResponse structure contains the ServiceData and AdditionalInfo which is unused
    currently.
    
    :var serviceData: The ServiceData
    :var additionalInfo: The additional info.
    """
    serviceData: ServiceData = None
    additionalInfo: AdditionalInfo = None


@dataclass
class PropertyNamePairValue(TcBaseObj):
    """
    A source and target property name pair that are equivalent and the associated source value.
    
    :var sourcePropName: Property  name of the equivalent property found on the original ItemRevision corresponding to
    the targetPropName.
    :var targetPropName: Equivalent property name for sourcePropName.
    :var sourceValue: The value of sourcePropName from the passed in original ItemRevision.
    """
    sourcePropName: str = ''
    targetPropName: str = ''
    sourceValue: str = ''


@dataclass
class AdditionalInfo(TcBaseObj):
    """
    AdditionalInfo has a list of key value pairs to capture the result.
    
    :var strToDateVectorMap: String to date vector map
    :var strToDoubleVectorMap: String to Double vector map
    :var strToStrVectorMap: String to String vector map
    :var strToObjVectorMap: String to Teamcenter BusinessObject vector map
    :var strToIntVectorMap: String to Integer vector map
    """
    strToDateVectorMap: StringToDateVectorMap = None
    strToDoubleVectorMap: StringToDoubleVectorMap = None
    strToStrVectorMap: StringToStringVectorMap = None
    strToObjVectorMap: StringToObjectVectorMap = None
    strToIntVectorMap: StringToIntegerVectorMap = None


@dataclass
class AlignLinesInBOMData(TcBaseObj):
    """
    Contains the details of an input for alignment.
    
    :var scopeElements: The list of BOMLine objects representing the scopes under which equivalent child BOMLine
    objects are aligned. This list is empty for single line alignment.
    :var equivalentLines: The list of EquivalentLinesPair structures. This list is not empty only for single line
    alignment.
    :var alignmentMode: String representing one of the four alignment modes: "AlignIdInTopLevelContextOnly" - single
    line alignment, "RecursivelyAlignAssemblies" - assemblies alignment, "AlignTransformOnReuseAssemblyOnly" - resued
    assemblies alignment, and "AlignMultipleNodesOfReuseAssm" - align all assigned lines under the reused assembly.
    :var alignProperties: If true, align all occurrence properties of the equivalent lines based on preference
    "MEAlignedPropertiesList".
    :var alignAllAssemblies: Applicable only to alignmentMode "AlignTransformOnReuseAssemblyOnly" or
    "AlignMultipleNodesOfReuseAssm". If true, indicates that in addition to aligning the selected reused assembly, the
    system also aligns the rest of the matching reused assemblies in the MBOM.
    :var additionalInfo: Additional input information; currently not used.
    """
    scopeElements: List[AlignmentScopeElement] = ()
    equivalentLines: List[EquivalentLinesPair] = ()
    alignmentMode: str = ''
    alignProperties: bool = False
    alignAllAssemblies: bool = False
    additionalInfo: AdditionalInfo = None


@dataclass
class AlignLinesInBOMResponse(TcBaseObj):
    """
    Contains the details of the alignment results.
    
    :var logTickets: The list of file tickets for the alignment logs. The size of logTickets is the same as that of the
    alignInputs list.
    :var summaries: The list of AlignmentSummary structures containing the summary information about the alignment. The
    size of summaries is the same as that of the alignInputs list.
    :var serviceData: The ServiceData containing partial errors.
    :var additionalInfo: Additional output information; currently not used.
    """
    logTickets: List[str] = ()
    summaries: List[AlignmentSummary] = ()
    serviceData: ServiceData = None
    additionalInfo: AdditionalInfo = None


@dataclass
class EquivalentLinesPair(TcBaseObj):
    """
    Contains a pair of equivalent BOMLine objects.
    
    :var sourceLine: The source BOMLine object to align with.
    :var targetLine: The target BOMLine object to be aligned.
    """
    sourceLine: BusinessObject = None
    targetLine: BusinessObject = None


@dataclass
class EquivalentPropertyNames(TcBaseObj):
    """
    List of property names and the property category they belong to.
    
    :var propertyCategory: A business object type to which property belong.  Valid values are:
    - Item
    - ItemRevision
    - Item Master
    - ItemRevision Master
    
    
    :var propertyNames: List of property names to find equivalent property values of  original ItemRevision.
    """
    propertyCategory: str = ''
    propertyNames: List[str] = ()


"""
A map of string to list of dates
"""
StringToDateVectorMap = Dict[str, List[datetime]]


"""
A map of string to list of doubles
"""
StringToDoubleVectorMap = Dict[str, List[float]]


"""
A map of string to list of integers
"""
StringToIntegerVectorMap = Dict[str, List[int]]


"""
A map of string to list of Teamcenter BusinessObjects
"""
StringToObjectVectorMap = Dict[str, List[BusinessObject]]


"""
A map of string to list of strings
"""
StringToStringVectorMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Awp0GatewayTileRel, Awp0Tile, UserSession
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetChildrenOutput(TcBaseObj):
    """
    GetChildrenOutput
    
    :var parentObject: parentObject
    :var children: children
    """
    parentObject: BusinessObject = None
    children: List[BusinessObject] = ()


@dataclass
class GetChildrenResponse(TcBaseObj):
    """
    GetChildrenResponse
    
    :var output: output
    :var serviceData: serviceData
    """
    output: List[GetChildrenOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GetCurrentUserGatewayResponse(TcBaseObj):
    """
    Contains the Awp0Tile, Awp0GatewayTileRel and the group name in which the Awp0Tile belongs to. The 'tileGroupInfos'
    are ordered based on the 'awp0OrderNo' on the Awp0GatewayTileRel.  For example, the group with the lowest
    'awp0OrderNo' value on the Awp0GatewayTileRel appears first in the list.
    
    :var tileGroupInfos: Information about the Awp0Tile and its group.
    :var serviceData: Awp0Tile and Awp0GatewayTileRel objects are added to the 'ServiceData' list of plain objects.
    """
    tileGroupInfos: List[CurrentUserTileGroupInfo] = ()
    serviceData: ServiceData = None


@dataclass
class GetTCSessionInfoResponse(TcBaseObj):
    """
    Data structure representing the different current user's Teamcenter session information.
    
    :var userSession: The Teamcenter user's session information.
    :var serviceData: The service data.
    """
    userSession: UserSession = None
    serviceData: ServiceData = None


@dataclass
class HistoryInput(TcBaseObj):
    """
    The input structure to add objects to the history and/or remove objects from the history.
    
    :var objectsToAdd: The list of objects to add to the history; this list can be empty.
    :var objectsToRemove: The list of objects to remove from the history; this list can be empty.
    :var returnHistory: Flag indicating whether the objects in the history are to be returned.
    """
    objectsToAdd: List[BusinessObject] = ()
    objectsToRemove: List[BusinessObject] = ()
    returnHistory: bool = False


@dataclass
class HistoryResult(TcBaseObj):
    """
    The response structure containing the result after adding objects to the history and/or removing objects from the
    history.
    
    :var historyObjects: The list of objects contained in the history.
    :var serviceData: The service data object.
    """
    historyObjects: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class LoadDataForEditingInfo(TcBaseObj):
    """
    Input structure of LoadDataForEditing SOA
    
    :var obj: Input object
    :var propertyNames: Property names
    :var isPessimisticLock: Flag indicating if object should be locked pessimisticly
    """
    obj: BusinessObject = None
    propertyNames: List[str] = ()
    isPessimisticLock: bool = False


@dataclass
class LoadDataForEditingOutput(TcBaseObj):
    """
    Output structure of LoadDataForEditing operation
    
    :var obj: Object to start edit or its dependent objects
    :var objLsds: Map of last save date of object or dependent objects
    """
    obj: BusinessObject = None
    objLsds: List[ObjectLsdInfo] = ()


@dataclass
class LoadDataForEditingResponse(TcBaseObj):
    """
    Response of LoadDataForEditing SOA
    
    :var outputs: Outputs
    :var serviceData: Service data
    """
    outputs: List[LoadDataForEditingOutput] = ()
    serviceData: ServiceData = None


@dataclass
class NameValueStruct(TcBaseObj):
    """
    This structure hold property name and vector of property values. This is a generic container that contains the
    property name as string and the value is the string representation of the property value.
    
    :var name: Property name string that needs to be modified.
    :var values: Property values string vector that will be used to update the input object with corresponding values.
    This vector contains string representation of the property value. The calling client is responsible for converting
    the different property types (int, float, date .etc) to a string using the appropriate toXXXString functions in the
    SOA client framework's Property class.
    """
    name: str = ''
    values: List[str] = ()


@dataclass
class ObjectLsdInfo(TcBaseObj):
    """
    Structure containing the lsd information for an input Business object.
    
    :var obj: Input Business object that needs to be saved.
    :var lsd: 'Last Modified date' (lsd) information of an input Business object that will be used before saving the
    modified properties of that input object. This is used to check the last modified time at the point of save, to
    ensure the right changes are applied to that object and that it has not been changed since the edit was started.
    """
    obj: BusinessObject = None
    lsd: datetime = None


@dataclass
class SaveEditAndSubmitInfo(TcBaseObj):
    """
    Structure represents the parameters required to save the edits on the input objects and submit them to a workflow
    process.
    
    :var object: Object to save edit and submit to a workflow process.
    :var propertyNameValues: Property name and values structure that will contain all property names and corresponding
    string values that needs to be saved.
    :var objLsds: The LSD information for the object.
    :var isPessimisticLock: Flag indicating object is pessimistically locked or not. If value is true, then object is
    locked while startEdit(), else object is not locked in startEdit() operation.
    :var workflowData: The workflow information that the input object needs to submit to a workflow or not.Workflow
    information is stored in a name and value string map that can contain property name as string and value will be
    also be string representation. Supported properties are  submitToWorkflow, processName, processDescription,
    processTemplate, processAssignmentList. 
    submitToWorkflow-  Boolean Property to define that input object need to submit to workflow process or not. It can
    contain true or false as value.
    processName- Process name string. 
    processDescription- Process description string. 
    processTemplate- Name of the process template to be used to create new workflow process. 
    processAssignmentList- Name of the process assignment list to use while creating new workflow process.
    Note- If the above property submitToWorkflow contains true value and process template value is empty, thenworkflow
    process template to be used for workflow creation should be specified in the preference
    (<TypeName>_default_workflow_template) defined for the submitted obejct type.
    """
    object: BusinessObject = None
    propertyNameValues: List[NameValueStruct] = ()
    objLsds: List[ObjectLsdInfo] = ()
    isPessimisticLock: bool = False
    workflowData: PropertyValues = None


@dataclass
class SaveEditAndSubmitOutput(TcBaseObj):
    """
    Structure containing information about each input object and whether it was saved and submitted to workflow
    successfully or not.
    
    :var clientId: Input string to uniquely identify the input, used primarily for error handling and output mapping.
    :var workflowProcess: Workflow template object created for each input object that was submitted to workflow.
    """
    clientId: str = ''
    workflowProcess: BusinessObject = None


@dataclass
class SaveEditAndSubmitResponse(TcBaseObj):
    """
    Structure containing saveEditAndSubmit() operation response.
    
    :var output: A list of SaveEditAndSubmitOutput structures.
    :var serviceData: Objects updated after save operation are added to ServiceData&rsquo;s updated object list.  
    Workflow processes created, after input objects are submitted to workflow, are added to ServiceData&rsquo;s created
    object list. Partial errors are returned in the Service Data.
    """
    output: List[SaveEditAndSubmitOutput] = ()
    serviceData: ServiceData = None


@dataclass
class SaveEditInfo(TcBaseObj):
    """
    Input structure of saveEdit SOA
    
    :var obj: Object to save edit
    :var propertyNameValues: Property name and values
    :var objLsds: Lsd map
    :var isPessimisticLock: Flag indicating object is pessimisticly locked in loadDataForEditing
    """
    obj: BusinessObject = None
    propertyNameValues: List[NameValueStruct] = ()
    objLsds: List[ObjectLsdInfo] = ()
    isPessimisticLock: bool = False


@dataclass
class CurrentUserTileGroupInfo(TcBaseObj):
    """
    Contains the Awp0Tile, Awp0GatewayTileRel, and the group name in which the Awp0Tile belongs. The 'tileInfos' are
    ordered based on the 'awp0OrderNo' property on the Awp0GatewayTileRel object.
    
    :var tileGroupName: The group name of the Awp0Tile.
    :var tileInfos: List of Awp0Tiles in this group.
    """
    tileGroupName: str = ''
    tileInfos: List[CurrentUserTileInfo] = ()


@dataclass
class CurrentUserTileInfo(TcBaseObj):
    """
    Contains the Awp0Tile and Awp0GatewayTileRel object. The size of the Awp0Tile and the order number within the group
    can be retrieved by inflating 'awp0TileSize' and 'awp0OrderNo' property on the Awp0GatewayTileRel object.
    
    :var tile: The Awp0Tile to be displayed.
    :var tileRel: The Awp0GatewayTileRel associating the Awp0Tile and Awp0TileCollection.
    """
    tile: Awp0Tile = None
    tileRel: Awp0GatewayTileRel = None


"""
Map (string, vector<string>) that is a generic container that represents property values. The key is the property name and the value is the string representation of the property value.
"""
PropertyValues = Dict[str, List[str]]



import dataclasses
import json
import logging
import sys
from enum import Enum
from typing import List, Dict, Iterable

import requests

from datetime import datetime

from tcsoa.config import TcSoaConfig
from tcsoa.exceptions import InvalidCredentialsException, InternalServerException, ServiceException, Severity, \
    InnerException
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBoClass


class RestBackend:
    def __init__(self, host: str):
        """
        Initializes the REST Backend. Should be useable for all Teamcenter Backends which have AWC installed.

        @param host: The address to your web server - for example: http://webserver:7001/tc
        """
        self.host = host
        self.base_url = f'{self.host}/RestServices'
        self.policy = {
            "useRefCount": False,
            "types": list()
        }
        self.state = {
            "stateless": True,
            "unloadObjects": True,
            "enableServerStateHeaders": True,
            "formatProperties": True,
            "clientID": "TcSoaClient"
        }
        self.session = requests.session()
        self.session.trust_env = False
        self.session.proxies = {
            "http": None,
            "https": None,
        }

    def _get_object_load_policy_for_bo(self, bo_name):
        types_list = self.policy['types']
        matching_type = list(t for t in types_list if t['name'] == bo_name)
        if matching_type:
            object_policy = matching_type[0]
        else:
            object_policy = {
                'name': bo_name
            }
            types_list.append(object_policy)
        return object_policy

    def set_object_load_policy(self, bo_name, properties: List[str]):
        object_policy = self._get_object_load_policy_for_bo(bo_name)
        object_policy['properties'] = [{'name': p} for p in properties]

    def add_object_load_policy(self, bo_name, properties: List[str]):
        object_policy = self._get_object_load_policy_for_bo(bo_name)
        objs_props = object_policy.setdefault('properties', [])
        for prop in properties:
            if not any(p for p in objs_props if p['name'] == prop):
                objs_props.append({'name': prop})

    def clear_object_load_policy(self, bo_name):
        self.policy['types'] = list(t for t in self.policy['types'] if t['name'] != bo_name)

    def clear_all_object_load_policies(self):
        self.policy['types'] = list()

    def input_obj_to_json(self, input_obj):
        if input_obj is None:
            return ''
        if type(input_obj) in (str, int, float, bool):
            return input_obj
        if isinstance(input_obj, TcBoClass):
            return {k: self.input_obj_to_json(v) for k, v in input_obj.__dict__.items() if k != 'props'}
        if dataclasses.is_dataclass(input_obj):
            return {k[:-1] if k.endswith('_') else k: self.input_obj_to_json(v) for k, v in input_obj.__dict__.items()}
        if isinstance(input_obj, datetime):
            return input_obj.strftime(TcSoaConfig.date_format)
        if isinstance(input_obj, Enum):
            return input_obj.value
        if isinstance(input_obj, dict):
            return {k: self.input_obj_to_json(v) for k, v in input_obj.items()}
        if type(input_obj) in (list, set, tuple) or isinstance(input_obj, Iterable):
            return [self.input_obj_to_json(o) for o in input_obj]
        raise NotImplementedError()

    def response_to_output_obj(self, response_cls, response):
        try:
            response_data: Dict[str: any] = response.json()
            qname = response_data.pop('.QName')
            if qname == 'http://teamcenter.com/Schemas/Soa/2006-03/Exceptions.InvalidCredentialsException':
                raise InvalidCredentialsException(**self._data_to_exception(response_data))
            if qname == 'http://teamcenter.com/Schemas/Soa/2006-03/Exceptions.ServiceException':
                raise ServiceException(**self._data_to_exception(response_data))
            if qname == 'http://teamcenter.com/Schemas/Soa/2006-03/Exceptions.InternalServerException':
                raise InternalServerException(**self._data_to_exception(response_data))

            if response_cls:
                response_data = self.dataclass_from_dict(response_cls, response_data)
            return response_data
        except ValueError:
            # note to self: when XML is returned, you dont have a valid cookie
            return response.content  # todo: handle this better

    def _data_to_exception(self, data):
        messages = data.get('messages', None)
        inner_exs = ()
        if messages:
            data = messages[0]
            inner_exs = [InnerException(**self._data_to_exception(m)) for m in messages[1:]]
        return dict(message=data['message'], code=data['code'], level=Severity(data['level']), inner_exceptions=inner_exs)

    @staticmethod
    def eval_str_type(cls, field):
        """ Black Magic - Evaluates the type annotation in context of the class """
        return eval(field, globals(), sys.modules[cls.__module__].__dict__)

    @staticmethod
    def dataclass_from_dict(cls, from_dict):
        try:
            fieldtypes = cls.__annotations__
        except AttributeError:
            if dataclasses.is_dataclass(cls):
                fieldtypes = dict()     # __annotations__ is only set to data classes with attributes
            elif getattr(cls, '__origin__', None) == dict and isinstance(from_dict, list):
                # special case: from_dict is a tuple, containing 2 lists, where the first list contains keys, and
                # the second list contains the values
                return {
                    RestBackend.dataclass_from_dict(cls.__args__[0], key):
                        RestBackend.dataclass_from_dict(cls.__args__[1], from_dict[1][idx])
                    for idx, key in enumerate(from_dict[0])}
            elif isinstance(from_dict, (tuple, list)):
                return [RestBackend.dataclass_from_dict(cls.__args__[0], f) for f in from_dict]
            elif isinstance(from_dict, dict):
                return {
                    RestBackend.dataclass_from_dict(cls.__args__[0], k):
                        RestBackend.dataclass_from_dict(cls.__args__[1], v)
                    for k, v in from_dict.items()
                }
            else:
                # logging.warning(f'Warning! Expected attribute class {str(cls)} but received {from_dict} !')
                return from_dict

        instance = cls()
        # if isinstance(from_dict, list) and len(from_dict) == 1 and isinstance(from_dict[0], dict):
        #     from_dict = from_dict[0]    # HACK! can't believe I have to do this! mainly for class `FileTicketsResponse`
        for f in from_dict:
            dest_f = f
            if f == 'ServiceData' and 'serviceData' in fieldtypes:
                dest_f = 'serviceData'

            if dest_f in fieldtypes:
                field = fieldtypes[dest_f]
            else:
                logging.debug(f'Warning! unregistered attribute "{f}" is tried to be set on class "{cls.__name__}"!')
                setattr(instance, dest_f, from_dict[f])
                continue

            if isinstance(field, str):  # black magic incoming
                field = RestBackend.eval_str_type(cls, field)
            setattr(instance, dest_f, RestBackend.dataclass_from_dict(field, from_dict[f]))
        if TcSoaConfig.global_obj_cache_enabled:
            if isinstance(instance, ServiceData):
                TcSoaConfig.internal_handle_sd(instance)
        return instance

    def execute(self, service_id, method_name, input_obj, response_cls):
        json_body = dict(
            header=dict(
                state=self.state,
                policy=self.policy,
            ),
            body=self.input_obj_to_json(input_obj)
        )
        url = f'{self.base_url}/{service_id}/{method_name}'
        data = json.dumps(json_body, ensure_ascii=False).encode('utf8')
        response = self.session.post(url, data=data, headers={
            'Operation-Name': method_name,
            'App-Xml': 'application/json',
            'Content-Type': 'application/json',
        })
        response.encoding = "utf-8"
        response_obj = self.response_to_output_obj(response_cls, response)
        return response_obj



from __future__ import annotations

from tcsoa.gen.Administration._2018_11.IRM import GetSessionInfoTicketResponse, GetSessionInfoFromTicketResponse
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService
from tcsoa.gen.Administration._2018_11.OrganizationManagement import UserConsentStatement


class IRMService(TcService):

    @classmethod
    def getSessionInfoFromTicket(cls, sessionInfoTicket: str) -> GetSessionInfoFromTicketResponse:
        """
        This operation gets all the session information in the form of key values map only if valid sessionInfoTicket
        is provided by client. Each key corresponds to particular session attribute like user, roles, groups, project
        teams, and licenses. For each entry in the keys array, the corresponding entry in the values array contains the
        values for that specific session attribute. Session information returned from this operation is used during
        read expression evaluation in external clients to determine the READ privilege to the current logged in user on
        the indexed Teamcenter data.
        
        Use cases:
        VDS (Visualization Data Server) has no user concept or connection authentication by design. TC SessionInfo
        passed to VDS clients after valid Teamcenter login but no validation is done by VDS today. Due to this
        shortcoming VDS is potential for spoofing, user session hacking and session reuse. To overcome this issue two
        new service operations are proposed and will be used. Calling client VDS uses getSessionInfoTicket operation to
        get encrypted (tamper proof) data packet and preserves the encrypted string. VDS calls SOA
        GetSessionInfoFromTicket with this ticket to get back the user session information. GetSessionInfoFromTicket
        returns back the session information only if the ticket is not expired.
        """
        return cls.execute_soa_method(
            method_name='getSessionInfoFromTicket',
            library='Administration',
            service_date='2018_11',
            service_name='IRM',
            params={'sessionInfoTicket': sessionInfoTicket},
            response_cls=GetSessionInfoFromTicketResponse,
        )

    @classmethod
    def getSessionInfoTicket(cls) -> GetSessionInfoTicketResponse:
        """
        This operation returns User, Group, Session Info, Site identifier and Ticket Expiry Time in the form of an
        encrypted string. Encrypted string ticket returned from this operation is used by getSessionInfoFromTicket
        operation to retrieve the session info.
        
        Use cases:
        VDS has no user concept or connection authentication by design. Teamcenter session information is passed to VDS
        clients after valid Teamcenter login but no validation is done by VDS today. Due to this shortcoming VDS is
        potential for spoofing, user session hacking and session reuse. To overcome this issue 2 new service operations
        are proposed and will be used. Calling client VDS uses getSessionInfoTicket to get encrypted (tamper proof)
        data packet and preserves the encrypted string. VDS client calls getSessionInfoFromTicket with this ticket to
        get back the user session information. getSessionInfoFromTicket returns back the session information only if
        the ticket is not expired and valid.
        
        Exceptions:
        >Partial errors
        """
        return cls.execute_soa_method(
            method_name='getSessionInfoTicket',
            library='Administration',
            service_date='2018_11',
            service_name='IRM',
            params={},
            response_cls=GetSessionInfoTicketResponse,
        )


class OrganizationManagementService(TcService):

    @classmethod
    def getUserConsentStatement(cls) -> UserConsentStatement:
        """
        This operation will return consent statement for the locale the user is logged in. If there is no consent
        statement found for the user&rsquo;s locale then master locale consent statement as per localization definition
        will be returned.
        """
        return cls.execute_soa_method(
            method_name='getUserConsentStatement',
            library='Administration',
            service_date='2018_11',
            service_name='OrganizationManagement',
            params={},
            response_cls=UserConsentStatement,
        )

    @classmethod
    def recordUserConsent(cls, userConsent: bool) -> ServiceData:
        """
        This operation records the user&rsquo;s consent to the General Data Protection Regulation (GDPR).
        """
        return cls.execute_soa_method(
            method_name='recordUserConsent',
            library='Administration',
            service_date='2018_11',
            service_name='OrganizationManagement',
            params={'userConsent': userConsent},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Cfg0ConfiguratorPerspective
from tcsoa.gen.Configurator._2014_12.ConfiguratorManagement import GetAdmissibilityResponse
from tcsoa.gen.Internal.Configurator._2014_12.ConfiguratorManagement import GetModelOptionCondInput, OverlapStateResponse, UpdateAdmissibilityInputList, OverlapStateInput, GetModelOptionCondResponse
from typing import List
from tcsoa.base import TcService


class ConfiguratorManagementService(TcService):

    @classmethod
    def getModelAndOptionConditions(cls, operationConfigPerspective: Cfg0ConfiguratorPerspective, inputStructs: List[GetModelOptionCondInput]) -> GetModelOptionCondResponse:
        """
        This operation splits the given input expressions into the list of model and option condition expressions.
        The business object type of the Cfg0AbsValue value objects, with which the input expressions are formed,
        divides these values into a set of Cfg0AbsModel objects and a set of Cfg0AbsOptionValue objects. This operation
        decomposes each input expression into a two lists. One expression list that is formed exclusively with
        Cfg0AbsModel objects and other expression list that is formed exclusively with Cfg0AbsOptionValue objects.
        
        Use cases:
        Input expression will be spitted into model and option condition expressions.
        Use Case1: 
        Example: Input expression - "Model=m1 OR Model=m2 AND Engine=e1" is decomposed into 2 lists with 2 elements
        each. 
        The model expression list will contain: modelExpressions[0]="Model=m1" 
                                                                         modelExpressions[1]= "Model=m2". 
        The option condition list will contain: optionConditions[0]= "" (an empty expression representing the Boolean  
                      constant TRUE)                                      
                                                                        optionConditions[1]="Engine=e1". 
        ORing the AND combinations of each model expression with the option condition at the same index forms an
        expression that is logically equivalent to the input expression:
        ModelExpression[0] &amp; OptionCondition[0] | ModelExpression[1] &amp; OptionCondition[1] == InputExpression
        Use Case2:
        Example: InputExpression= "OF1 = V1 AND (ModelFamily = M1 OR ModelFamily = M3) AND OF3 = V3)"
        The Output will contain two list corresponding to the input expression. One list will correspond to the model
        expression and the other list will correspond to the option condition expression.
        ModelExpression will contain:  modelExpressions[0]="ModelFamily=M1"
                                                               modelExpressions[1]="ModelFamily=M3"
        Variant condition list will contain: optionConditions[0]= " OF1=V1 &amp; OF3=V3"
                                                                  optionConditions[1]= " OF1=V1 &amp; OF3=V3"
        
        Exceptions:
        >The operation will return a ServiceException if an unknown error occurs. A possible source for unknown errors
        could be a codeful configurator component customization.
        """
        return cls.execute_soa_method(
            method_name='getModelAndOptionConditions',
            library='Internal-Configurator',
            service_date='2014_12',
            service_name='ConfiguratorManagement',
            params={'operationConfigPerspective': operationConfigPerspective, 'inputStructs': inputStructs},
            response_cls=GetModelOptionCondResponse,
        )

    @classmethod
    def getOverlapStates(cls, context: Cfg0ConfiguratorPerspective, overlapStateInputs: List[OverlapStateInput]) -> OverlapStateResponse:
        """
        This operation determines and returns the degree of overlap between a set of configuration expressions and a
        reference configuration expression.
        
        Use cases:
        An application wants to qualify variant conditions that were retrieved with getVariantExpressions as to whether
        the condition is equal to, intersects with, or is a subset or superset of the variant criteria associated with
        the currently active variant configuration criteria (VariantRule). This cannot be achieved with properties on
        variant conditions or VariantRules because the result depends on the combination of a variant condition and the
        variant configuration criteria on a VariantRule. One and the same condition may have different overlap states
        with different VariantRules.
        The application calls getOverlapStates and passes the variant criteria (as obtained from a VariantRule using
        getVariantExpressions) as referenceExpression, and the variant conditions (as obtained from a set of product
        data elements using getVariantExpressions) as expressions.
        """
        return cls.execute_soa_method(
            method_name='getOverlapStates',
            library='Internal-Configurator',
            service_date='2014_12',
            service_name='ConfiguratorManagement',
            params={'context': context, 'overlapStateInputs': overlapStateInputs},
            response_cls=OverlapStateResponse,
        )

    @classmethod
    def updateAdmissibility(cls, updateAdmissibilityInputList: List[UpdateAdmissibilityInputList]) -> GetAdmissibilityResponse:
        """
        This operation updates the admissibility state for the families in context of the input context object. The
        admissibility of a target object such as family is defined as whether that target object is available or not
        available for the given context object such as partition..:
        The instance of Cfg0AbsAdmissibility object represents the relation and state between a configurator specific
        object and a non-configurator specific object. For example, the pair of a partition and a family is associated
        with state "Available". This operation will help to update the state for this pair to "Not Available".
        If the input state provided is blank then this operation will delete the corresponding Cfg0AbsAdmissibility
        object and related objects such as Cfg0AbsAdmissibilityThread.
        
        Use cases:
        This operation can be used to update admissibility for the families for the given business object.
         
        Consider that following set data-
        
        Engine-Box (A Family Group) - It has families "Engine" and "Transmission". 
        1. Engine- Petrol, Diesel
        2. Transmission - Manual, Auto
        Wheel (An Option Family Group) - It has families "Wheel-drive" and "Suspension".
        1. Wheel-drive - 2-Wheels, 4-Wheels
        2. Suspension - Full-Thrust, Full-Boom
        For the Engine partition instance, the families Engine and Transmission are "Available" while the families
        Wheel-drive and Suspension are "Not Available".
        If user wants to update the admissibility of Wheel-drive for Engine Partition as "Available" then using
        updateAdmissibility operation the admissibility state can be updated.
        
        Exceptions:
        >No error conditions result at this time for this operation.
        """
        return cls.execute_soa_method(
            method_name='updateAdmissibility',
            library='Internal-Configurator',
            service_date='2014_12',
            service_name='ConfiguratorManagement',
            params={'updateAdmissibilityInputList': updateAdmissibilityInputList},
            response_cls=GetAdmissibilityResponse,
        )



import logging
from random import randint

from deprecated import deprecated
from typing import List, Dict

from tcsoa.backend import RestBackend
from tcsoa.config import TcSoaConfig
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Core._2010_09.DataManagement import PropInfo, NameValueStruct1
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.Core._2007_12.Session import StateNameValue
from tcsoa.gen.Core._2011_06.Session import Credentials
from tcsoa.gen.Core.services import DataManagementService, SessionService


class TcConnection:
    def __init__(self, url: str, username: str, password: str, descrimator: str = None, insecure: bool = False):
        """
        This class aims to provide a simple interface to create a Teamcenter connection.
        Use it either like so:
            >>> conn = TcConnection('http://my.teamcenter:7001/tc', 'dcproxy', 'dcproxy')
            >>> try:
            >>>     # do TC stuff
            >>> finally:
            >>>     conn.close()
        Or with the Context Manager syntax like this:
            >>> with TcConnection('http://my.teamcenter:7001/tc', 'dcproxy', 'dcproxy'):
            >>>     # do TC stuff
        Or - if you are fearless - just like that
            >>> TcConnection('http://my.teamcenter:7001/tc', 'dcproxy', 'dcproxy')
            >>> # do TC stuff
            >>> TcSoaBasics.logout()


        :param url: Your Teamcenter URL, like 'http://my.teamcenter:7001/tc'
        :param username: The Teamcenter username to be used to connect to Teamcenter
        :param password: The Teamcenter password of the username provided to connect to Teamcenter
        :param descrimator: The descrimator to use - if not provided, a random descriminator will be used
        :param insecure: A flag for convenience to disable TLS checking
        """
        self.url = url
        self.username = username
        self.password = password
        self.descrimator = descrimator
        self.insecure = insecure

        self.connect()

    def __enter__(self):
        pass

    def connect(self):
        TcSoaConfig.backend = RestBackend(self.url)
        if self.insecure:
            TcSoaConfig.backend.session.verify = False
            import urllib3
            urllib3.disable_warnings()

        TcSoaBasics.setCredentials(self.username, self.password)
        TcSoaBasics.descrimator = self.descrimator if self.descrimator else f'TcSoa_{randint(0, 999999)}'
        login_result = TcSoaBasics.login()
        login_success = len(login_result['PartialErrors']) == 0
        if login_success:
            logging.info(f'Logged in on Server {TcSoaBasics.server} - with log file {TcSoaBasics.log_file}')
            return self
        else:
            errors = '\n'.join(login_result['PartialErrors'])
            logging.error('Error occured when logging in:\n' + errors)
            raise ConnectionError(errors)

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def close(self):
        TcSoaBasics.logout()


class TcSoaBasics:
    NULLTAG = 'AAAAAAAAAAAAAA'

    # Login configuration
    username = None
    password = None
    group = ''
    role = ''
    locale = 'en_US'
    descrimator = 'TcSoa.py'

    # Server Information
    log_file: str = None
    server: str = None

    @classmethod
    def setCredentials(cls, username, password, group='', role=''):
        cls.username = username
        cls.password = password
        cls.group = group
        cls.role = role

    @classmethod
    def login(cls):
        login_response = SessionService().login3(
            credentials=Credentials(
                user=cls.username,
                password=cls.password,
                group=cls.group,
                role=cls.role,
                locale=cls.locale,
                descrimator=cls.descrimator,
            )
        )
        cls.log_file = login_response.serverInfo['LogFile']
        cls.server = login_response.serverInfo['HostName']
        return login_response

    @classmethod
    def logout(cls):
        return SessionService().logout()

    @classmethod
    def load_objects(cls, uids: List[str]) -> Dict[str, BusinessObject]:
        if not uids:
            return dict()
        load_objs_result = DataManagementService().loadObjects(uids=uids)
        model_objects = load_objs_result['modelObjects']
        return {p: model_objects[p] for p in load_objs_result['plain']}

    @classmethod
    def load_object(cls, uid: str):
        obj = None
        for obj in cls.load_objects([uid]).values():
            break
        return obj

    @classmethod
    def getProperties(cls, objects: List[BusinessObject], properties: List[str]) -> Dict[str, BusinessObject]:
        response = DataManagementService().getProperties(
            objects=objects,
            attributes=properties
        )
        return {uid: response['modelObjects'][uid] for uid in response['plain']}

    @classmethod
    @deprecated(version="0.5.0", reason="Use the other getPropXXXX methods instead.")
    def getProperty(cls, bo: BusinessObject, prop_name: str):
        return cls.getProperties([bo], [prop_name])[bo['uid']]['props'][prop_name]

    @classmethod
    def getPropString(cls, bo: BusinessObject, prop_name: str):
        lst = cls.getPropStringList(bo, prop_name)
        return lst[0] if lst else None

    @classmethod
    def getPropStringList(cls, bo: BusinessObject, prop_name: str):
        return cls.getProperties([bo], [prop_name])[bo['uid']]['props'][prop_name].get('uiValues', list())

    @classmethod
    def getMultiPropStringList(cls, bos: List[BusinessObject], prop_name: str) -> Dict[str, List[str]]:
        return {uid: obj.prop(prop_name) for uid, obj in cls.getProperties(bos, [prop_name]).items()}

    @classmethod
    def getMultiPropString(cls, bos: List[BusinessObject], prop_name: str) -> Dict[str, str]:
        return {uid: val_list[0] if val_list else None for uid, val_list in cls.getMultiPropStringList(bos, prop_name).items()}

    @classmethod
    def getPropObject(cls, bo: BusinessObject, prop_name: str) -> BusinessObject:
        lst = cls.getPropObjectList(bo, prop_name)
        return lst[0] if lst else None

    @classmethod
    def getPropObjectList(cls, bo: BusinessObject, prop_name: str) -> List[BusinessObject]:
        response: ServiceData = DataManagementService().getProperties(
            objects=[bo],
            attributes=[prop_name],
        )
        ref_obj_uids = response.modelObjects[bo.uid].prop(prop_name, db_value=True)
        return [response.modelObjects[uid] for uid in ref_obj_uids if uid]

    @classmethod
    def getMultiPropObjectList(cls, bos: List[BusinessObject], prop_name: str) -> Dict[str, List[BusinessObject]]:
        response = DataManagementService().getProperties(
            objects=bos,
            attributes=[prop_name],
        )
        obj2refs_map = {bo.uid: response.modelObjects[bo.uid].prop(prop_name, db_value=True) for bo in bos}
        return {uid: [response.modelObjects[ref] for ref in refs if ref] for uid, refs in obj2refs_map.items()}

    @classmethod
    def getMultiPropObject(cls, bos: List[BusinessObject], prop_name: str) -> Dict[str, BusinessObject]:
        return {uid: val_list[0] if val_list else None for uid, val_list in cls.getMultiPropObjectList(bos, prop_name).items()}

    @classmethod
    def bool2property_str(cls, val: bool) -> str:
        return '1' if val else '0'

    @classmethod
    def setMultiPropString(cls, bo: BusinessObject, prop2val_map: Dict[str, str]):
        DataManagementService.setProperties2(
            info=[
                PropInfo(
                    object=bo,
                    vecNameVal=[
                        NameValueStruct1(
                            name=prop_name,
                            values=[prop_val]
                        )
                        for prop_name, prop_val in prop2val_map.items()
                    ]
                )
            ],
            options=[],
        )

    @classmethod
    def setPropString(cls, bo: BusinessObject, prop_name: str, prop_val: str):
        DataManagementService.setProperties2(
            info=[
                PropInfo(
                    object=bo,
                    vecNameVal=[
                        NameValueStruct1(
                            name=prop_name,
                            values=[prop_val]
                        )
                    ]
                )
            ],
            options=[],
        )

    @classmethod
    def setBypass(cls, enabled=True):
        return SessionService().setUserSessionState(
            pairs=[
                StateNameValue(
                    name='fnd0bypassflag',
                    value=cls.bool2property_str(enabled)
                )
            ]
        )

    @classmethod
    def set_object_load_policy(cls, bo_name: str, properties: List[str]):
        TcSoaConfig.backend.set_object_load_policy(bo_name, properties)

    @classmethod
    def add_object_load_policy(cls, bo_name: str, properties: List[str]):
        TcSoaConfig.backend.add_object_load_policy(bo_name, properties)

    @classmethod
    def clear_object_load_policy(cls, bo_name: str):
        TcSoaConfig.backend.clear_object_load_policy(bo_name)

    @classmethod
    def clear_all_object_load_policies(cls):
        TcSoaConfig.backend.clear_all_object_load_policies()

    @classmethod
    def get_cached_obj(cls, uid: str):
        obj = TcSoaConfig.internal_get_obj(uid)
        if obj is not None:
            bo = BusinessObject()
            bo.uid = bo.objectID = obj['uid']
            bo.type = obj['type']
            bo.className = obj['className']
            return bo
        return None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMWindow
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService
from tcsoa.gen.Internal.StructureManagement._2015_10.VariantManagement import VariantConfigurationCriteria


class VariantManagementService(TcService):

    @classmethod
    def applyVariantConfiguration(cls, window: BOMWindow, variantConfigurationCriteria: VariantConfigurationCriteria) -> ServiceData:
        """
        The applyVariantConfiguration operation configures  the Product Structure in the window by input formula or
        variant rule. The value of PSM_enable_product_configurator  preference must be true for successful completion
        of this operation.
        
        Use cases:
        Use Case 1:  Configure the window by applying formula or variant rule-  
        
        You can get formula from getVariantExpressions operation or can pass it in the following format -
        "[FamilyNamespace]FamilyName=FamilyValue. For example [Teamcenter]Color = red | [Teamcenter]Color = blue". This
        will configure the window accordingly or you can configure the window using variant rule created in Product
        Configurator application. 
        
         
        
        Use Case 2:  Configure the window by applying formula or variant rule with defaults and rule checks  - 
        
        To fully configure the BOMWindow for variants, you need to evaluate defaults and rule checks which can be done
        via below operations. You can extract variant criteria as formula string from either getProductDefaults or from
        validateProductConfiguration depending on the requirement. Typically getProductDefaults and
        validateProductConfiguariton in sequence and can pass final variant criteria as input for
        applyVariantConfiguartion.
        
        
        Order of service operation calls -
        - getVariantExpression
        - getProductDefaults
        - validateProductConfiguration
        - applyVariantConfiguration
        
        """
        return cls.execute_soa_method(
            method_name='applyVariantConfiguration',
            library='Internal-StructureManagement',
            service_date='2015_10',
            service_name='VariantManagement',
            params={'window': window, 'variantConfigurationCriteria': variantConfigurationCriteria},
            response_cls=ServiceData,
        )



from tcsoa.gen.Internal_CodeCoverage._2011_06.services import CodeCoverageManagementService as imp0
from tcsoa.base import TcService


class CodeCoverageManagementService(TcService):
    performOperation = imp0.performOperation



from __future__ import annotations

from tcsoa.gen.Cad._2008_06.StructureManagement import AbsOccDataPref, CreateOrUpdateAbsoluteStructurePref3, CreateOrUpdateAbsoluteStructureInfo3, AbsOccQualifierInfo, ExpandPSAllLevelsResponse2, ReconfigureBOMWindowsInfo, ExpandPSAllLevelsPref, CreateVariantRulesInfo, CreateOrUpdateAbsoluteStructureResponse2, CreateVariantRulesResponse, ExpandPSOneLevelPref, ExpandPSOneLevelResponse2, DeleteRelativeStructureInfo3, ReconfigureBOMWindowsResponse, SaveBOMWindowsResponse, CreateOrUpdateRelativeStructureInfo3, ExpandPSOneLevelInfo, ExpandPSAllLevelsInfo, GetAbsoluteStructureDataResponse, CreateOrUpdateRelativeStructurePref3
from tcsoa.gen.BusinessObjects import BOMWindow, Folder
from tcsoa.gen.Cad._2007_12.DataManagement import CreateOrUpdatePartsPref
from tcsoa.gen.Cad._2008_06.DataManagement import CreateOrUpdatePartsResponse, PartInfo, ExpandFoldersForCADResponse2, ExpandFolderForCADPref2
from tcsoa.gen.Cad._2007_01.StructureManagement import CreateOrUpdateRelativeStructureResponse, DeleteRelativeStructureResponse
from tcsoa.gen.Cad._2007_12.StructureManagement import DeleteRelativeStructurePref2
from typing import List
from tcsoa.base import TcService


class StructureManagementService(TcService):

    @classmethod
    def reconfigureBOMWindows(cls, info: List[ReconfigureBOMWindowsInfo]) -> ReconfigureBOMWindowsResponse:
        """
        This operation takes a list of BOMWindow objects and updates the contents of the windows (i.e. configuration)
        by applying the supplied RevisionRule and variant configuration information.  If the
        'RevisionRuleEntryProps'::'unitNo' is set to -1 then it considers default unitNo or use the input RevisionRule
        object with no changes. If no value specified for 'RevisionRuleEntryProps'::'unitNo', then the input
        RevisionRule object used as modified/transient rule with unitNo as 0. If the value of preference
        PSM_enable_product_configurator is set to true, then Product Configurator variant rule will be honored.
        
        Use cases:
        This operation is used to reconfigure the BOMWindow with new or modified RevisionRule and VariantRule
        information.
        """
        return cls.execute_soa_method(
            method_name='reconfigureBOMWindows',
            library='Cad',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'info': info},
            response_cls=ReconfigureBOMWindowsResponse,
        )

    @classmethod
    def saveBOMWindows(cls, bomWindows: List[BOMWindow]) -> SaveBOMWindowsResponse:
        """
        This operation can be used by a client developer to save any modifications made to a Teamcenter product
        structure through its runtime artifacts. A Teamcenter product structure is a persistent parent child
        composition and clients often deal with runtime artifacts of this persistent model. The runtime artifacts are
        primarily represented by BOMLine business objects and BOMWindow business objects along with the BOMLine
        properties. Modifications to the product structure are typically bulked up on the client and tracked through
        the BOMWindow. Once it is established that changes to a product structure can be saved this operation can be
        called passing in a handle to the BOMWindows that need to be saved.
        
        Use cases:
        In a typical usage of the saveBOMWindows operation, a client developer already has a Teamcenter product
        structure loaded with the runtime artifacts created. This means that the client developer has/creates handles
        to a BOMWindow, and BOMLine business objects that are part of that BOMWindow. Given this pre disposition,
        typical interaction with the client may involve addition of a new BOMLine or removal of a BOMLine,
        creation/removal of In Structure associations etc. In such cases, the changes to the product structure are
        tracked on the BOMWindow and when the changes are ready to be persisted, the client developer calls the
        saveBOMWindows operation.
        """
        return cls.execute_soa_method(
            method_name='saveBOMWindows',
            library='Cad',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'bomWindows': bomWindows},
            response_cls=SaveBOMWindowsResponse,
        )

    @classmethod
    def createOrUpdateAbsoluteStructure(cls, absOccInfos: List[CreateOrUpdateAbsoluteStructureInfo3], bomViewTypeName: str, complete: bool, pref: CreateOrUpdateAbsoluteStructurePref3) -> CreateOrUpdateAbsoluteStructureResponse2:
        """
        This operation creates or updates absolute occurrences on an existing relative structure.  The structure will
        be configured with revision rule set in the preference TC_config_rule_name before any configuration changes are
        applied. The objects created or updated can include an absolute occurrence, assembly arrangement and datasets
        and are added to the ServiceData as created or updated objects.  Created forms are added as plain objects and
        updated forms are added as updated objects.
        
        Before creating the absolute occurrence or assembly arrangement, this operation will check whether either
        already exists.  If the occurrence or arrangement exists but the input attribute values differ from those
        already set, an attempt is made to update the values.
        This operation assumes the input is in a bottom up format such that if any failures occur, the structure that
        is updated will still be consumable.  For example:
        
        Parent assembly A and occurrence C are input into this operation along with subassembly B and occurrence D.  If
        occurrence D is updated successfully on subassembly B but an error occurs during the update of occurrence C on
        assembly A, the client can still access subassembly B.
        
        For objects of type dataset and form, this operation can create or update a new object instance and attach it
        as an override on the absolute occurrence.  For all object types, existing objects can be attached or removed
        as an override on the absolute occurrence.
        
        To help reduce the client overhead of sending all override information during a complete synchronization and
        avoid accidental removal of overrides coming from other clients, this operation allows the client to provide
        only the attributes it is interested in syncing when the complete flag is set to true.  For more information on
        complete synchronization, see the description for the complete input parameter.
        
        One parent context object is processed at a time and it is assumed that all edits for a given parent context
        come in as one set of input.
        
        Use cases:
        Use case 1:
        
        User adds an override to an existing assembly to create an absolute occurrence.
        
        For this operation, the assembly is passed in as the 'contextItemRev' and the override, such as new
        transformation data to position a component in an assembly, is passed in as the absolute occurrence.  The
        absolute occurrence is created and a map of the input 'clientID' to AbsOccurrence is returned in 'absOccOutput'
        and the occurrence is returned as created objects in 'ServiceData'.
        
        Use case 2:
        
        User adds an override with a new dataset to an existing assembly to create an absolute occurrence with an
        attached dataset.
        
        For this operation, the assembly is passed in as the 'contextItemRev' and the override with information for the
        new dataset is passed in as the absolute occurrence information.  The absolute occurrence and new dataset are
        created and a map of the input 'clientID' to AbsOccurrence is returned in 'absOccOutput' and the created
        dataset is returned in 'datasetOutput'.  The occurrence and dataset are returned as created objects in
        'ServiceData'.
        
        
        Exceptions:
        >Service Exception    Thrown if an invalid BOM view type is specified in 'bomViewTypeName'.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateAbsoluteStructure',
            library='Cad',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'absOccInfos': absOccInfos, 'bomViewTypeName': bomViewTypeName, 'complete': complete, 'pref': pref},
            response_cls=CreateOrUpdateAbsoluteStructureResponse2,
        )

    @classmethod
    def createOrUpdateRelativeStructure(cls, inputs: List[CreateOrUpdateRelativeStructureInfo3], bomViewTypeName: str, complete: bool, pref: CreateOrUpdateRelativeStructurePref3) -> CreateOrUpdateRelativeStructureResponse:
        """
        This is the overloaded function for Cad::_2007_12::StructureManagement::createOrUpdateRelativeStructure.
        It differs by allowing the parent member of the structure CreateOrUpdateRelativeStructureInfo to be a
        ModelObject instead of a strongly typed Item Revision object and makes the child member of the
        RelativeStructureChildInfo to a ModelObject instead of strongly typed Item Revision.
        This is to allow structures with GDE elements to be edited with this service.
        Create or update the relative structure objects and relations for the input model.
        The service first attempts to check for the presence of duplicate context objects and then validate the
        existence of the structure to be created.
        If any of the objects exist but the input attribute values that differ from those already set, an attempt is
        made to update the values if possible.
        This service assumes the input is in a bottom-up format such that if any failures occur, the structure that is
        created will be consistent.
        No attempt is made in the service to rearrange the input and process it in the correct order.
        As we process one parent context object at one time, it is assumed that all edits for a given parent context
        come in as one set of input.
        
        Exceptions:
        >Service exceptions are thrown for the following error situations:
        
        215005: The BOM view revision (BVR) is not checked out by the current user.
        215006: The BOM view revision is checked out by another user.
        215009: The object type name specified in the input preference does not exist.
        215010: The input CAD occurrence identifier does not correspond to a valid BOM line. This is an error when the
        complete input is false.
        215033: The input last modified date differs from actual.
        215034: The dataset or BVR was modified even when the input last modified dates was different than the current
        last modified data. 
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateRelativeStructure',
            library='Cad',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'inputs': inputs, 'bomViewTypeName': bomViewTypeName, 'complete': complete, 'pref': pref},
            response_cls=CreateOrUpdateRelativeStructureResponse,
        )

    @classmethod
    def createVariantRules(cls, input: List[CreateVariantRulesInfo]) -> CreateVariantRulesResponse:
        """
        This operation creates the saved VariantRule objects for classic variant options.
        
        Use cases:
        This operation is used to create VariantRule object and save them, which can be used later for BOM variant
        configuration.
        """
        return cls.execute_soa_method(
            method_name='createVariantRules',
            library='Cad',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=CreateVariantRulesResponse,
        )

    @classmethod
    def deleteRelativeStructure(cls, inputs: List[DeleteRelativeStructureInfo3], bomViewTypeName: str, pref: DeleteRelativeStructurePref2) -> DeleteRelativeStructureResponse:
        """
        This operation deletes one or more first level children of a parent assembly.  The children, or occurrences, to
        be deleted are specified by the CAD occurrence ID or PSOccurrenceThread UID.  The last modified date of the BOM
        view revision (BVR) can also be specified for comparison against the last modified date in Teamcenter to ensure
        the occurrence has not changed outside of this client context and control if the BVR is actually modified and
        the occurrence deleted by this operation.
        
        Use cases:
        User deletes an existing relative occurrence from an existing assembly.
        
        For this operation, the assembly is passed in as the parent and the occurrence ID is passed in as the child
        information.  The relative occurrence is removed from the parent assembly and the ID of the deleted occurrence
        is added to the ServiceData list of deleted objects.
        
        Exceptions:
        >Service Exception    Thrown if an invalid BOM view type is specified in 'bomViewTypeName'.
        """
        return cls.execute_soa_method(
            method_name='deleteRelativeStructure',
            library='Cad',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'inputs': inputs, 'bomViewTypeName': bomViewTypeName, 'pref': pref},
            response_cls=DeleteRelativeStructureResponse,
        )

    @classmethod
    def expandPSAllLevels(cls, info: ExpandPSAllLevelsInfo, pref: ExpandPSAllLevelsPref) -> ExpandPSAllLevelsResponse2:
        """
        This is the overloaded function for Cad::_2007_01::StructureManagement::expandPSAllLevels.
        The member itemRevOfBOMLine of ExpandPSData and ExpandPSParentData is renamed as objectOfBOMLine and its type
        is changed from Item Revision to ModelObject.
        Also the datasets and parentDatasets member of ExpandPSData and ExpandPSParentData structures respectively are
        renamed to relatedObjects and type changed to ModelObject instead of Dataset.
        This is to support structures with GDE elements to be returned from the expansion.
        Finds the children at all levels given parent BOMLines.
        Also if required gets the objects of given type and relation that are attached to the parent and children.
        In addition to the above, the expansion of the Product Structure can be filtered for a given occurrence type/s
        which can be included and/or excluded from the expansion.
        In addition to the above the following additional functionality has been added:
        1.  The operation is not limited to return related objects of dataset type only.  Expansion of attached objects
        to the BOM line object is determined by a filtering mechanism where the criteria is set by: relation name,
        related object type, and named references.
        2.  This operation allows for expansion to reference object associated to a named reference.  Typically this is
        a file.  An FMS ticket will be returned to provide access to this file.
        3.  Where a named reference points to a file, this operation allows for specific logic in choosing which files
        are returned.  This is determined by the input parameter NamedRefHandler (included in the info object).
        """
        return cls.execute_soa_method(
            method_name='expandPSAllLevels',
            library='Cad',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'info': info, 'pref': pref},
            response_cls=ExpandPSAllLevelsResponse2,
        )

    @classmethod
    def expandPSOneLevel(cls, info: ExpandPSOneLevelInfo, pref: ExpandPSOneLevelPref) -> ExpandPSOneLevelResponse2:
        """
        This operation will expand one level of a product structure and return the resulting BOM lines.
        If required, it will return objects of a given relation and type that are attached to the parent and child BOM
        lines.
        Additionally, if specified in the preference, this will return only BOM Lines of a given particular occurrence
        type and exclude occurrence types of a given type.
        This operation differs from the operation Teamcenter.Soa.Cad._2007_01.StructureManagement.expandPSOneLevel in
        the following ways:
        1.  The operation is not limited to return related objects of dataset type only.  Expansion of attached objects
        to the BOM line object is determined by a filtering mechanism where the criteria is set by: relation name,
        related object type, and named references.
        2.  This operation allows for expansion to reference object associated to a named reference.  Typically this is
        a file.  An FMS ticket will be returned to provide access to this file.
        3.  Where a named reference points to a file, this operation allows for specific logic in choosing which files
        are returned.  This is determined by the input parameter NamedRefHandler (included in the info object).
        4.  The resulting Product Structure expansion can be limited to a certain Occurrence Types.
        """
        return cls.execute_soa_method(
            method_name='expandPSOneLevel',
            library='Cad',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'info': info, 'pref': pref},
            response_cls=ExpandPSOneLevelResponse2,
        )

    @classmethod
    def getAbsoluteStructureData(cls, absOccDataQualInfos: List[AbsOccQualifierInfo], absOccDataPref: AbsOccDataPref) -> GetAbsoluteStructureDataResponse:
        """
        This operation retrieves the absolute occurrence override information for the given qualifier.
        The input accepts the relation override that needs to be expanded.
        In case of finding the overrides based on the used arrangement within the structure, the client is expected to
        provide bvr of sub assembly in the input.
        ParentBVR |--(Arr1) |--(Arr2) | |________childBVR1 |           |-----(Arr3) |           |-----(Arr4) |         
         |_____________childBVR2 |                                    |----(Arr5) |                                   
        |----(Arr6) |_____ childBVR2 |----(Arr5) |----(Arr6)
        If Arr1 uses Arr3 which in turn uses Arr5, to determine the overrides by Arr3 and Arr5 qualifier the client is
        expected to send childbvr1 and childbvr2 as inputs along with the parentBVR.
        """
        return cls.execute_soa_method(
            method_name='getAbsoluteStructureData',
            library='Cad',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'absOccDataQualInfos': absOccDataQualInfos, 'absOccDataPref': absOccDataPref},
            response_cls=GetAbsoluteStructureDataResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def createOrUpdateParts(cls, partInput: List[PartInfo], pref: CreateOrUpdatePartsPref) -> CreateOrUpdatePartsResponse:
        """
        CreateOrUpdateParts allows the user to update or create a set of parts using Items, Item Revisions, and
        Datasets and save the boudingbox information related to these objects.
        The service first attempts to validate the existence of the Item, Item Revision, and Dataset.
        If the Item and Item Revision already exist, but the Dataset does not, then only the Dataset is created.
        If the Dataset exists, a new version will be added to the existing series.
        If any of the objects exist but the input attribute values that differ from those already set, attempts are
        made to update the values if possible.
        If the boundingbox information exists it saves that information on that particular dataset else it willnot save
        the boudingbox information.
        If no item object reference or item revision object references are specified then a new item and item revision
        and related objects will be created.
        All objects created and updated will be returned in the ServiceData.
        All partial errors will contain the clientIDs for all items related to the error message, i.e. if a dataset
        encounters an error, then the ID for that erro will be the item client ID concantentated with the item revision
        id contantenated with the dataset client ID, all separated by semi-colons ( ; ).
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateParts',
            library='Cad',
            service_date='2008_06',
            service_name='DataManagement',
            params={'partInput': partInput, 'pref': pref},
            response_cls=CreateOrUpdatePartsResponse,
        )

    @classmethod
    def expandFoldersForCAD(cls, folders: List[Folder], pref: ExpandFolderForCADPref2) -> ExpandFoldersForCADResponse2:
        """
        The purpose of this service is to provide the contents of a folder that a CAD integration typically needs in
        one service call as opposed to multiple expand calls. This service is specifically for folder expansion and
        will only return items, item revisions and folders that are contained in the input folder. Other types of
        objects ( forms, datasets, etc.. ) that are contained directly under the input folder can be returned if their
        type is specified in the preference. The service will also return the item revisions and datasets for the items
        found in the folder and datasets found for the item revisions found directly under the folder. The item
        revisions returned, are controlled thru an input 'latestNRevs' specifying how many latest revisions should be
        sent to output. The items, item revisions and datasets returned can be filtered thru an input preference of a
        list of relation names and dataset types filter.
        
        Use cases:
        A CAD application needs to know what objects in the database are relative to the cad application. The CAD
        application can get the contents of the home folder and display to the user. If the user then selects a
        subfolder of home then the application can expand that folder to get to the desired data. This service will
        return objects that the cad application understands such as subfolders, items, item revisions and datasets and
        other objects of the type the application specifies in the input filter.
        """
        return cls.execute_soa_method(
            method_name='expandFoldersForCAD',
            library='Cad',
            service_date='2008_06',
            service_name='DataManagement',
            params={'folders': folders, 'pref': pref},
            response_cls=ExpandFoldersForCADResponse2,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, Item, PSViewType, ItemRevision
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ReplaceItemsParameter(TcBaseObj):
    """
    Input structure for the operation 'replaceItems'. The structure contains selected BOMLine, information related to
    replacement Item and the replace option.
    
    :var bomLine: The selected BOMLine to do the replace operation.
    :var item: The replacement Item.
    :var itemRevision: The replacement ItemRevision.
    :var viewType: The view type to be used in replace. It determines the BOMView Revision to be used for the
    occurrence of the ItemRevision. If the replacement ItemRevision has no BOMView Revision, uses NULLTAG .
    For additional information, please see the Structure Manager Guide.
    :var replaceOption: Replacement options.
    - 0 for replacing the selected occurrence.
    - 1 for replacing all sibling occurrences of the selected in the parent assembly. 
    - 2 for replacing all occurrences of the selected in the entire structure.
    
    """
    bomLine: BOMLine = None
    item: Item = None
    itemRevision: ItemRevision = None
    viewType: PSViewType = None
    replaceOption: int = 0



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ConfigurationFamily, POM_object, RevisionRule
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ProductInfo(TcBaseObj):
    """
    The ProductInfo structure represents product name and product namespace combination. This structure is interpreted
    as NULL if it has empty ( ) values for 'productName' and 'productNameSpace' parameters.
    
    :var productName: Specifies a product. A ('productName,productNamespace') tuple is used as a parameter in
    configurator service calls so to allow the configurator to resolve any ambiguity in effectivity condition
    parameters. Applications using this 'EffectivityManagement' service should define a mechanism to obtain an
    appropriate value for this parameter. For example, 'ApplicationModel' objects like Collaborative Design models
    (Cpd0CollaborativeDesign) make them available via properties 'mdl0config_product_name', and
    'mdl0config_prod_namespace.'
    :var productNameSpace: Specifies a namespace in which parameter 'productName' has a unique definition, e.g. a
    specific model year, or product type. A ('productName,productNamespace') tuple is used as a parameter in
    configurator service calls so to allow the configurator to resolve any ambiguity in effectivity condition
    parameters. Applications using this 'EffectivityManagement' service should define a mechanism to obtain an
    appropriate value for this parameter. For example, 'ApplicationModel' objects like Collaborative Design models
    (Cpd0CollaborativeDesign) make them available via properties 'mdl0config_product_name', and
    'mdl0config_prod_namespace.'
    """
    productName: str = ''
    productNameSpace: str = ''


@dataclass
class AvailableProductEffectivityResponse(TcBaseObj):
    """
    Response structure containing available product effectivity information.
    
    :var configExpressions: Vector of expressions specifying an available range of effectivity. Together they cover the
    range of availability over all families given in the input parameters 'familiesToTest' and (indirectly)
    'exprsToTest'.
    :var serviceData: Partial errors are returned in Service Data.
    """
    configExpressions: List[ConfigExpression] = ()
    serviceData: ServiceData = None


@dataclass
class RevRuleEffectivityCriteria(TcBaseObj):
    """
    The RevRuleEffectivityCriteria structure represents a revision rule along with its effectivity criteria and
    associated validation records. The RevRuleEffectivityCriteria structure is interpreted to be NULL if the following
    values are set for its parameters:
    - revisionRule: NULLTAG
    - primaryEffectivity: a NULL ConfigFormula structure with all its parameters set to empty strings
    - solveType: neg 1
    - validationRecords: empty vector
    
    
    
    :var revisionRule: The modified RevisionRule. This could be a transient copy.
    :var primaryEffectivity: Original unprocessed effectivity criteria, before effectivity configurator rules were
    applied.
    :var solveType: Indicates the type of solve that will be performed when using the effectivity criteria in
    configuration filters.
    :var validationRecords: Validation records related to the effectivity criteria
    """
    revisionRule: RevisionRule = None
    primaryEffectivity: ConfigFormula = None
    solveType: int = 0
    validationRecords: List[EffectivityValidationRecord] = ()


@dataclass
class RevRuleEffectivityCriteriaResponse(TcBaseObj):
    """
    Response structure containing Revision Rule Effectivity Criteria Infomation.
    
    :var effectivityCriteria: Vector of effectivity criteria structures.
    :var serviceData: Partial errors are returned in Service Data.
    """
    effectivityCriteria: List[RevRuleEffectivityCriteria] = ()
    serviceData: ServiceData = None


@dataclass
class ConfigExpression(TcBaseObj):
    """
    The 'ConfigExpression' structure is interpreted as 'NULL' if it has the following values:
    - subExpressions: an empty vector
    - value: a 'NULL' 'ConfigOption' structure
    - opCode: neg 1
    - formula: a 'NULL' 'ConfigFormula' structure
    - effectivityTable: an empty vector
    
    
    
    :var subExpressions: The list of sub expressions to be combined with 'opcode'. 'subExpressions' and value must not
    both be present. If neither 'subExpressions' nor 'value' are present, the expression is treated as the identity
    element for all 'opcode' values.
    :var value: The value to which 'opCode' compares. 'subExpressions' and value must not both be present. If neither
    'subExpressions' nor 'value' are present, the expression is treated as the identity element for all 'opcode' values.
    :var opCode: - The operation code such as 'or', and, or 'equal'. See '$TC_INCLUDE/ps/ps_tokens.h.'
    
    
    :var formula: If present, contains the configuration expression as a configuration formula.
    :var effectivityTable: If present, contains a representation of the configuration expression formatted as a table
    where each row represents one validity range.
    """
    subExpressions: List[ConfigExpression] = ()
    value: ConfigOption = None
    opCode: int = 0
    formula: ConfigFormula = None
    effectivityTable: List[EffectivityTableRow] = ()


@dataclass
class ConfigFormula(TcBaseObj):
    """
    The 'ConfigFormula' structure represents a formula string associated with a configuration expression in
    configurator syntax. The 'ConfigFormula' structure is considered NULL if it has following values set for its
    parameters:
    - formula: empty string ( )
    - productName: empty string ( )
    - productNameSpace: empty string ()
    
    
    
    :var formula: Formula string in a configurator specific encoding. Teamcenter configurators used in the
    EffectivityManagement service interface encode formula in Explicit Teamcenter Language.
    :var productName: Specifies a product (e.g. using its Item ID). May be used in conjunction with parameter
    'productNameSpace' when resolving ambiguity in effectivity option value names.
    :var productNameSpace: Namespace in which parameter 'productName' has a unique definition, e.g. an Item Revision
    ID, a Model Year, or a Product Type. May be used in conjunction with parameter 'productNameSpace' when resolving
    ambiguity in effectivity option value names.
    """
    formula: str = ''
    productName: str = ''
    productNameSpace: str = ''


@dataclass
class ConfigOption(TcBaseObj):
    """
    The ConfigOption structure represents a configuration option that is unique in the context of a configuration
    family. The ConfigOption structure is interpreted as "NULL" if it has the following values set for its parameters:
     - family: NULLTAG
     - name: empty string ("")
     - description: empty string ("")
    
    :var family: Namespace in which this option has unique semantics.
    :var name: Name of the option value. "" is treated as NULL, i.e., it does not match any criteria, not even other
    NULL values.
    :var description: Description of the option value.
    """
    family: ConfigurationFamily = None
    name: str = ''
    description: str = ''


@dataclass
class ConfigRuleViolation(TcBaseObj):
    """
    The ConfigRuleViolation structure represents a violated rule along with the message associated with rule violation.
    The ConfigRuleViolation structure is interpreted to be NULL if following values are set to its parameters:
    - message: empty string ( )
    - violatedCondition: a NULL ConfigExpression structure
    
    
    
    :var message: The text string associated with the rule violation.
    :var violatedCondition: The rule that was found to be violated.
    """
    message: str = ''
    violatedCondition: ConfigExpression = None


@dataclass
class ConfigurableProductsResponse(TcBaseObj):
    """
    Response structure containing Configurable Product Information.
    
    :var configurableProducts: List of configurable products defined in the effectivity configurator.
    :var serviceData: Partial errors are returned in ServiceData.
    """
    configurableProducts: List[ProductInfo] = ()
    serviceData: ServiceData = None


@dataclass
class DefaultRule(TcBaseObj):
    """
    The DefaultRule structure represents the details of the default expression that is applied to a config condition.
    The DefaultRule structure is interpreted to be NULL if it has following values set to its parameters:
    - partiallyApplicable: false
    - restrictiveCondition: a NULL ConfigExpression structure
    - appliedDefault: NULL ConfigExpression structure
    - appliedTo: NULL ConfigExpression structure
    
    
    
    :var partiallyApplicable: If true, the DefaultRule is only applicable to a subset of the criteria.
    :var restrictiveCondition: The condition under which the default is applicable. A NULL ConfigExpression structure
    indicates that the rule is always applicable.
    :var appliedDefault: The default that is applied, e.g., whenever restrictiveCondition is met.
    :var appliedTo: The configuration criteria before applying the default.
    """
    partiallyApplicable: bool = False
    restrictiveCondition: ConfigExpression = None
    appliedDefault: ConfigExpression = None
    appliedTo: ConfigExpression = None


@dataclass
class EffectivityCondition(TcBaseObj):
    """
    The EffectivityCondition structure is interpreted to be "NULL" if the following values are set to its parameters:
    formula: a "NULL" ConfigFormula structure (with all its parameters sets to empty strings); affectedElement: NULLTAG
    
    :var formula: Effectivity condition formula
    :var affectedElement: Product element on which the effectivity condition has been set
    """
    formula: ConfigFormula = None
    affectedElement: POM_object = None


@dataclass
class EffectivityConditionResponse(TcBaseObj):
    """
    Response structure returning a set effectivity expressions. The response structure is returned from multiple
    operations and needs to be interpreted depending on the context in which it is returned.
    
    :var effectivityConditions: A vector of effectivity condition structures where each element references the
    resulting effectivity condition and the corresponding object. The return vector does not necessarily need to
    correspond to the input vector by index as it may need to contain information about additional modified objects
    along with their new effectivity condition. This could be the result of effectivity propagation to sub elements.
    :var serviceData: Partial errors are returned in Service Data.
    """
    effectivityConditions: List[EffectivityCondition] = ()
    serviceData: ServiceData = None


@dataclass
class EffectivityExpressionsResponse(TcBaseObj):
    """
    Response structure returning a set effectivity expressions. The response structure is returned from multiple
    operations and needs to be interpreted depending on the context in which it is returned.
    
    :var effectivityExpressions: A vector of effectivity expressions. The response structure is returned from multiple
    operations and needs to be interpreted depending on the context in which it is returned.
    :var serviceData: Partial errors are returned in Service Data.
    """
    effectivityExpressions: List[ConfigExpression] = ()
    serviceData: ServiceData = None


@dataclass
class EffectivityFormulaeResponse(TcBaseObj):
    """
    Response structure returning a set configuration formulae in a configurator specific encoding. The response
    structure is returned from multiple operations and needs to be interpreted depending on the context in which it is
    returned.
    
    :var effectivityFormulae: A vector configuration formulae each representing one effectivity expression in a
    configurator specific encoding. The response structure is returned from multiple operations and needs to be
    interpreted depending on the context in which it is returned.
    :var serviceData: Partial errors are returned in Service Data
    """
    effectivityFormulae: List[ConfigFormula] = ()
    serviceData: ServiceData = None


@dataclass
class EffectivityTable(TcBaseObj):
    """
    The 'EffectivityTable' structure represents the set of validity ranges for a configuration expression.
    If 'EffectivityTables' for different expressions (e.g. Unit = 5 and Unit > 4 & Unit 'EffectivityTableRows' might be
    different, but will still be logically equivalent to the input expressions they correspond to.
    The following example illustrates 3 different logically equivalent 'EffectivityTable' structures. A given
    Teamcenter server session will consistently return one of them.
    
    - Unit=1..2 Date=Jan..Feb
    - Unit=1..2 Date=Feb..Mar
    - Unit=1..2 Date=Mar..Apr
    - Unit=2..3 Date=Mar..Apr
    - Unit=3..4 Date=Mar..Apr 
    
    
    
    The effectivity table above is equivalent to
    
    - Unit=1..2 Date=Jan..Apr
    - Unit=2..4 Date=Mar..Apr
    
    
    
    But it is also equivalent to
    
    - Unit=1..4 Date=Mar..Apr
    - Unit=1..2 Date=Jan..Mar
    
    
    
    The 'EffectivityTable' structure is interpreted to be NULL if the following values are set to its parameters:
    
    - 'effectivityTableRows': an empty vector
    
    
    
    :var effectivityTableRows: A vector of EffectivityTableRow structures each representing an effectivity range. Some
    ranges may overlap.
    """
    effectivityTableRows: List[EffectivityTableRow] = ()


@dataclass
class EffectivityTableRow(TcBaseObj):
    """
    The EffectivityTableRow structure represents a validity range. The following constants have special meaning:
    - January 2, 1900 12:00 AM UTC: Open Start Date
    - December 30, 9999 12:00 AM UTC: Open End Date
    - December 26, 9999 12:00 AM UTC: Stock Out
    - 1: Open Start Unit
    - 2147483647: Open End Unit
    - 2147483646: Stock Out
    
    
    Effect in as well as effect out points may have NULL values, which indicate no value assigned:
    -     Neg 1: no unit value assigned
    -     NULL date: no date value assigned
    
    
    The 'EffectivityTableRow' structure is interpreted as NULL if the following values are set:
    -     unitIn: Neg 1
    -     unitOut: Neg 1
    -     dateIn: a NULL date
    -     dateOut: a NULL date
    -     rest: a NULL 'ConfigFormula' structure
    
    
    
    :var unitIn: Unit at which this validity range starts.
    :var unitOut: Unit at which the validity range ends.
    :var dateIn: Date at which the validity range starts.
    :var dateOut: Date at which the validity range ends.
    :var rest: Effectivity conditions in this range that are neither date nor unit related.
    """
    unitIn: int = 0
    unitOut: int = 0
    dateIn: datetime = None
    dateOut: datetime = None
    rest: ConfigFormula = None


@dataclass
class EffectivityTablesResponse(TcBaseObj):
    """
    'EffectivityTablesResponse' structure
    
    :var effectivityTables: A vector of effectivity table structures each representing one effectivity expression.
    :var serviceData: Partial errors are returned in Service Data
    """
    effectivityTables: List[EffectivityTable] = ()
    serviceData: ServiceData = None


@dataclass
class EffectivityValidationRecord(TcBaseObj):
    """
    The EffectivityValidationRecord structure represents validation results for effectivity criteria associated with a
    revision rule. The EffectivityValidationRecord structure is interpreted to be "NULL" if the following values are
    set to its parameters:
     - validationDate: a null date representing "0001-01-01T00:00:00Z" ("January 1, 0001, midnight, UTC")
     - validationResult: a "NULL" ConfigFormula structure with all its parameters set to empty strings
     - undeterminedFamilies: an empty vector
     - violations: an empty vector
     - appliedDefaults: an empty vector
    
    :var validationDate: Date at which the validation was performed.
    :var validationResult: Validation result after applying effectivity configurator rules.
    :var undeterminedFamilies: Families that are not set to a single value. If empty, the criteria are complete.
    :var violations: Violation records. If empty, the effectivity criteria are valid.
    :var appliedDefaults: One record for each applied default in the sequence they were applied
    """
    validationDate: datetime = None
    validationResult: ConfigFormula = None
    undeterminedFamilies: List[ConfigurationFamily] = ()
    violations: List[ConfigRuleViolation] = ()
    appliedDefaults: List[DefaultRule] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Item, ItemRevision
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class CloningStructsInfo(TcBaseObj):
    """
    Contains BOMLine objects which are to be cloned in CPC along with additional cloning information.
    
    :var newName: The name of new structure to be created as a cloned structure in CPC.
    :var newDescription: The description of new structure to be created as a cloned structure in CPC.
    :var newId: ID of new structure to be created as a cloned structure in CPC.
    :var newRevId: The revision ID of new structure to be created as a cloned structure in CPC.
    :var scopes: The list of  BOMLine objects to be cloned in CPC. It can be either root BOMLine or multiple BOMLine
    objects within the same structure.
    :var cloningRule: The rule used for cloning. The value varies based on the structure type which is getting cloned.
    For example, the acceptable value for Mfg0MEPlantBOP is  "ProjectPlantBOPTemplate".
    :var cloningParams: A map (string, string) containing cloning parameter as key and its flag as the value, such as 
    ("CPC_carryOverICs","true") for carry over Increamental Changes. Other cloning parameters are
    "CPC_cloneSupressLine" and "CPC_createDIPA" for clone suppressed lines and create Dynamic In-Process Assembly
    (DIPA) respectively.
    """
    newName: str = ''
    newDescription: str = ''
    newId: str = ''
    newRevId: str = ''
    scopes: List[BusinessObject] = ()
    cloningRule: str = ''
    cloningParams: CloningParams = None


@dataclass
class CreateCPCInputInfo(TcBaseObj):
    """
    A list of BOMLine objects  that are to be cloned and\or referred, cloning information, name and description of new
    CPC.
    
    :var cloningStructsInfo: A list of CloningStructureInfo which contains BOMLine objects which are to be cloned in
    CPC. The structure contains additional cloning information as well.
    :var releaseStatus: The release status which gets associated with line(s) in the cloned structure. If the line(s)
    doesn't have any release status in the original structure. It is configurable but in this version only acceptable
    value is "Alternate Planning Initial".
    :var effectivityInfo: The information required to set effectivity for a list of cloned BOMLine objects.
    :var originalCCUid: The UID of the MECollaborationContext from which CPC is to be created.
    :var cpcName: Name of the CPC to be created.
    :var cpcDesc: Description of the CPC to be created.
    :var refStructures: List of  BOMLine objects from the original CC that are going to be referred in the CPC.
    """
    cloningStructsInfo: List[CloningStructsInfo] = ()
    releaseStatus: str = ''
    effectivityInfo: EffectivityInfoInput = None
    originalCCUid: BusinessObject = None
    cpcName: str = ''
    cpcDesc: str = ''
    refStructures: List[BusinessObject] = ()


@dataclass
class CreateCPCResponse(TcBaseObj):
    """
    Contains newly created CPC data.
    
    :var cpcObject: The created CPC object.
    :var serviceData: The Service Data
    """
    cpcObject: BusinessObject = None
    serviceData: ServiceData = None


@dataclass
class EffectivityInfoInput(TcBaseObj):
    """
    The information required to set effectivity for a list of cloned BOMLine objects.
    
    :var endItem: Effectivity end Item.
    :var endItemRev: Effectivity end ItemRevision.
    :var unitRangeText: Effectivity unit range, for example "5-10". Multiple range can also be provided, for example
    "3,5-10,20-50". This range belongs to the end Item which is part of the structrue on which the unit effectivity is
    applied.
    Valid unit ranges are:
     -StartUnit
    -StartUnit - EndUnit
    -StartUnit - SO
    -StartUnit - UP
            where, StartUnit <= EndUnit 
    -StartUnit1 - EndUnit1, StartUnit2 - EndUnit2(Ex: 10 - 12, 15, 16 - UP)
            where StartUnit2 > EndUnit1.
    - All units are positive integers
    
    For example, the effectivity can be 3 - 5 where 3 is start unit and 5 is end unit.
    It can also be 3 - UP where 3 is start unit and "UP" means anything above the start unit. "UP" and "SO" indicates
    open ended effectivity.  "SO" is stock out means effectivity will be applicable till the stock out condition.
    :var dateRange: The list of effectivity date range. For instance, opened date range could be specified as "05 Jan -
    UP" and closed date range as "01 Jan - 03 Apr". The list may have any number of opened and closed date range. For
    example, If you select multiple dates such as start date 19th May 2015 and End date 22nd May 2015 , start date 19th
    June 2015 and end date 22nd June 2015, the dataRange list have 4 values as below : 2015-02-19, 2015-05-22,
    2015-06-19,2015-06-22.
    :var openEndedStatus: Effectivity open ended status.
    - 0 : for closed unit range effectivity or date effectivity.
    - 1 : for opened unit range effectivity or date effectivity (UP).
    - 2 : for opened unit range effectivity or date effectivity stock out (SO).
    For example, if you select open ended effectivity such as start date 19th May 2015 and end date "UP", the dateRange
    list has 2015-05-19 with "openEndedStatus" value as "1".
    """
    endItem: Item = None
    endItemRev: ItemRevision = None
    unitRangeText: str = ''
    dateRange: List[datetime] = ()
    openEndedStatus: int = 0


"""
Map of the cloning parameters. The key of the map is cloning parameter and its value is a flag.
"""
CloningParams = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Internal.Manufacturing._2016_09.StructureManagement import LinkStructuresResponse, DesignPartAlignmentInput, DesignPartAlignmentResponse, AsyncCreateCPCInputInfo, ObjectAlignmentInput, LinkStructuresInput, CreateDesignPartAlignmentInput, CompletenessCheckPartStructureResp
from tcsoa.gen.BusinessObjects import BOMLine, BusinessObject
from tcsoa.gen.Internal.Manufacturing._2016_09.IPAManagement import DynamicIPAInputInfo, SaveDynamicIPALinesResponse, DynamicIPALinesResponse, CleanDynamicIPALinesResponse
from tcsoa.gen.Internal.Manufacturing._2016_09.DataManagement import CreateBOEfromPlantBOPResponse, SynchronizeBOPAndBOEInputInfo, SynchronizePlantBOPAndBOEResponse, LinkPlantBOPtoBOEResponse, LinkBOPtoBOEObjectInfo
from tcsoa.gen.Internal.Manufacturing._2016_09.ResourceManagement import GetVendorCatalogInfo3Response
from typing import List
from tcsoa.gen.Internal.Manufacturing._2016_09.StructureSearch import SearchBOEInputInfo, SearchBOEResponse
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class IPAManagementService(TcService):

    @classmethod
    def cleanDynamicIPALines(cls, inputLines: List[DynamicIPAInputInfo]) -> CleanDynamicIPALinesResponse:
        """
        The operation cleans DIPA(Mfg0BvrDynamicIPA) lines generated using SOA createDynamicIPALines for the input BOP
        lines.
        
        Use cases:
        Use case 1 : Recursively clean DIPA(Mfg0BvrDynamicIPA) nodes on top line of process structure
        
        You can invoke this operation on top line of process structure. This operation  cleans  all the
        DIPA(Mfg0BvrDynamicIPA)nodes uder all the sub-processes.  
        
        Use case 2 : Recursively clean DIPA(Mfg0BvrDynamicIPA) nodes on intermediate processes in process structure.
        
        You can invoke this operation  on specific processeses.
        This operation cleans DIPA(Mfg0BvrDynamicIPA) node for input processes and each of its sub-processes.
                   
        Use case 3: Clean DIPA nodes on one or more intermediate processes of process structure
        
        You can invoke this operation on one or more processes of process structure.This operation cleans
        DIPA(Mfg0BvrDynamicIPA) node for the input processes only.
        """
        return cls.execute_soa_method(
            method_name='cleanDynamicIPALines',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='IPAManagement',
            params={'inputLines': inputLines},
            response_cls=CleanDynamicIPALinesResponse,
        )

    @classmethod
    def saveContentOfDynamicIPALines(cls, inputDIPALines: List[BusinessObject]) -> SaveDynamicIPALinesResponse:
        """
        This operation persists the content of the input dynamic IPA (Mfg0BvrDynamicIPA) nodes to the database.
        
        Use cases:
        Use case 1 : Save one or more Dynamic IPA(Mfg0BvrDynamicIPA)
        You can invoke this operation for one or more dynamic IPA nodes below different processes.
        saveContentOfDynamicIPALines saves the children of DIPA nodes into the database and also sets the 'Is
        Persistent'(fnd0IsPersistent) property to true.
        """
        return cls.execute_soa_method(
            method_name='saveContentOfDynamicIPALines',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='IPAManagement',
            params={'inputDIPALines': inputDIPALines},
            response_cls=SaveDynamicIPALinesResponse,
        )

    @classmethod
    def updateDynamicIPALines(cls, inputLines: List[DynamicIPAInputInfo]) -> DynamicIPALinesResponse:
        """
        This operation updates all the DIPA (Mfg0BvrDynamicIPA) nodes for input processes; and for all its
        sub-processes in a hierachy, if the recursive flag is true.
        
        Use cases:
        Use case 1 : Update on top line of the process structure
         
        You can invoke this operation on top line of the process structure. This operation updates all the     
        DIPA(Mfg0BvrDynamicIPA) nodes under all the sub-processes. Recursive flag is ignored for the top line.
        
        Use case 2 : Update on one or more DIPA (Mfg0BvrDynamicIPA) nodes
        
        You can invoke this operation on specific DIPA (Mfg0BvrDynamicIPA) nodes. This operation updates only the
        DIPA(Mfg0BvrDynamicIPA) nodes provided as input.
        
        Use case 3 : Update recursive on one or more intermediate processes in the process structure
        
        You can invoke this operation on one or more processes in the process structure. This operation                
                updates DIPA(Mfg0BvrDynamicIPA)  nodes for input processes and each of its sub-process only if the
        recursive flag is true.
        """
        return cls.execute_soa_method(
            method_name='updateDynamicIPALines',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='IPAManagement',
            params={'inputLines': inputLines},
            response_cls=DynamicIPALinesResponse,
        )

    @classmethod
    def createDynamicIPALines(cls, inputLines: List[DynamicIPAInputInfo]) -> DynamicIPALinesResponse:
        """
        This operation creates the DIPA (Mfg0BvrDynamicIPA) containing the incoming parts as a flat list and adds it to
        the process structure. If the input process has more than one predecessors, it creates a separate DIPA node for
        each incoming path. If the recursive flag is true, it creates DIPA for every process having predecessors in the
        hierarchy of the input process. The operation considers occurrence types defined in
        'MEDynamicIPAOccurrenceTypes' preference; to determine which occurrence types to aggregate from predecessors in
        the given process structure to generate the DIPA (Mfg0BvrDynamicIPA). The default value for this preference
        will be 'MEConsumed'. 
        These new DIPA (Mfg0BvrDynamicIPA) nodes  are added under the appropriate process with 'MEDynamicWorkpiece'
        occurrence type.
        
        Use cases:
        Use case 1 : Create DIPA(Mfg0BvrDynamicIPA)on one or more processes in process structure.You can invoke this
        operation on one or more processes. If the recursive flag is false, the operation  
        creates DIPA (Mfg0BvrDynamicIPA) nodes only below the input processes with 'MEDynamicWorkpiece'occurrence type. 
        
        Use case 2 : Create Recursive DIPA(Mfg0BvrDynamicIPA) nodes for top line of process structure.You can invoke
        this operation on top line of process structure.It creates DIPA(Mfg0BvrDynamicIPA) nodes for all the sub-
        processes having predecessors.
        
        Use case 3 : Create Recursive DIPA(Mfg0BvrDynamicIPA) for intermediate processes in process sructure.You can
        invoke the operation on onr or more intermediate processes in process structure.This operation creates
        DIPA(Mfg0BvrDynamicIPA)  nodes for input processes and each of its sub-processes having predecessors.
        """
        return cls.execute_soa_method(
            method_name='createDynamicIPALines',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='IPAManagement',
            params={'inputLines': inputLines},
            response_cls=DynamicIPALinesResponse,
        )


class ResourceManagementService(TcService):

    @classmethod
    def getVendorCatalogInfo3(cls, catalogTypeFilter: int) -> GetVendorCatalogInfo3Response:
        """
        The multi-value preference "MRMGTCVendorCatalogRootDir" specifies one or more root directories where vendor
        tool catalogs may be stored on the Teamcenter server machine.
        This operation retrieves additional information about valid vendor catalogs contained in these root
        directories. It scans the given root directories for vendor catalogs of the requested type and returns detailed
        information for each valid catalog. This new operation supports GTC packages in GTC V1 and V2 format.
        """
        return cls.execute_soa_method(
            method_name='getVendorCatalogInfo3',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='ResourceManagement',
            params={'catalogTypeFilter': catalogTypeFilter},
            response_cls=GetVendorCatalogInfo3Response,
        )

    @classmethod
    def extractHolderData(cls, icoIds: List[str]) -> ServiceData:
        """
        This operation extracts tool holder data from Manufacturing Resource Manager resources by using the NX
        Graphicsbuilder functionality.  The NX Graphicsbuilder is called to extract the holder data from the given
        resources.  The resulting holder data is stored in the resources' assembly ICOs.
        """
        return cls.execute_soa_method(
            method_name='extractHolderData',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='ResourceManagement',
            params={'icoIds': icoIds},
            response_cls=ServiceData,
        )


class StructureManagementService(TcService):

    @classmethod
    def linkOrUnlinkStructures(cls, input: List[LinkStructuresInput]) -> LinkStructuresResponse:
        """
        This operation links or unlinks the PSBOMView objects of the  two input structures by the input relation.
        """
        return cls.execute_soa_method(
            method_name='linkOrUnlinkStructures',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=LinkStructuresResponse,
        )

    @classmethod
    def completenessCheckPartStructure(cls, input: List[ObjectAlignmentInput]) -> CompletenessCheckPartStructureResp:
        """
        This operation does a completeness check of the Part structure.  The operation recursively checks all the Part
        objects in the structure for completeness.  Part objects not marked as design required are marked complete.  If
        the Part is marked as design required the following checks are performed:
         
        - Is Part structure connected?
        -     Checks for relation Fnd0PartCadRelation. 
        - Are Part and Design revision aligned?
        -     Checks the Part object's child DesignRevision against the Part object's configured DesignRevision.
        - Is Part occurrence an End Item?
        - Is schema valid?
        - Does Design occurrence thread exist?
        - Does Design occurrence exist?
        - Are Part and Design revision connected properly?
        - Checks for Fnd0PartCadRelation.
        - Is Part BVR precise?
        
        """
        return cls.execute_soa_method(
            method_name='completenessCheckPartStructure',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=CompletenessCheckPartStructureResp,
        )

    @classmethod
    def removeDesignPartAlignment(cls, input: List[ObjectAlignmentInput]) -> DesignPartAlignmentResponse:
        """
        This service operation removes Design and Part objects alignment if it exist.  
        It will be the reversal of the what is done by the createDesignPartAlignment operation as described below:
        - Remove precise BOMViewRevision (BVR) indication on Part (if needed)
        - Remove Design occurrence from Part.
        - Remove relation between Part and Design revisions.
        - Remove End Item designation from Part.
        - Remove Represented By / Represented For relations for Part and Design.
        - Clear bl_abs_occ_id (IDIC) from Part.
        
        """
        return cls.execute_soa_method(
            method_name='removeDesignPartAlignment',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=DesignPartAlignmentResponse,
        )

    @classmethod
    def createAsyncCollabPlanningContext(cls, asyncCpcInput: AsyncCreateCPCInputInfo) -> None:
        """
        This service operation asynchronously creates a Collaboration Planning Context (CPC) with the given input
        structures that are to be cloned and/or referred to, and establishes a relation between input
        MECollaborationContext (CC) object and newly created CPC. CPC is basically a CC object itself, it is a term
        used for Alternative Management. This is asynchronous implementation for the
        Teamcenter::Soa::Manufacturing::_2015_10::StructureManagement createCollabPlanningContext service operation.
        
        Use cases:
        A user can asynchronously create a CPC object in Manufacturing Process Planner (MPP) application using an
        existing opened CC structure. Subsequently a relation Mfg0MEAlternatePlanningRel is created between newly
        created CPC and the original CC.
        Use Case 1: Open a CC structure, select some structures available in the CC and create a CPC asynchronously.
        
        Use Case 2: Open a CC structure, select some of the scopes in that structure and create a CPC asynchronously.
        
        Use Case 3: Open a CC structure, select some structures/scopes, provide whether they need to be referred or
        cloned and provide cloning parameters such as clone suppressed lines, carry-over ICs etc. to create a CPC
        asynchronously.
        """
        return cls.execute_soa_method(
            method_name='createAsyncCollabPlanningContext',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='StructureManagement',
            params={'asyncCpcInput': asyncCpcInput},
            response_cls=None,
        )

    @classmethod
    def verifyDesignPartAlignment(cls, input: List[DesignPartAlignmentInput]) -> DesignPartAlignmentResponse:
        """
        This operation does a series of verification checks against an existing Design and Part object alignment.   It
        returns as partial errors any of the validations that fail.  The validation performed is as follows:
        
        - Part structure is connected
        - Checks for Fnd0PartDesignLink relation
        - Part and Design revision aligned 
        - Checks Part child DesignRevision against Part configured DesignRevision
        - Is Part occurrence End Item?
        - Is schema valid?
        - Does Design occurrence thread exist?
        - Does Design occurrence exist?
        - Part and Design revision connected properly
        - Checks for Fnd0PartCadRelation relation
        - Is Part BOMViewRevision (BVR) precise?
        
        """
        return cls.execute_soa_method(
            method_name='verifyDesignPartAlignment',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=DesignPartAlignmentResponse,
        )

    @classmethod
    def createOrUpdateDesignPartAlignment(cls, input: List[CreateDesignPartAlignmentInput]) -> DesignPartAlignmentResponse:
        """
        This service operation is used to create or update Part and Design object alignments between Design and Part
        structures. It is possible to create two different types of alignments. One type of alignment includes
        assignment of the Design occurrence to the Part occurrence. The other alignment type does not include this
        Design occurrence assignment. The alignment with Design occurrence assignment includes all the steps of the
        other type of alignment with additional steps involving the Design occurrence alignment. The common create
        alignment steps are described here:
        
        Create matching bl_abs_occ_id (IDIC) between Design and Part occurrences.
        Copy BOMLine properties from Design occurrence to Part occurrences.
        The preference EDesignPartAlignProperties describes the properties to copy.
        Create relation of type internal name between objects DesignRevision and PartRevision.
        The PartRevision being primary and DesignRrevision as secondary.
        The Part and Design TC_Is_Represented_By relation is established.
        Part BOMLine property alignment link exist indicator is set true.
        
        Additional steps for create alignment with Design occurrence assignment:
        
        Assign Design occurrence under Part occurrence
        Mark Part BOMViewRevision (BVR) as Precise
        Mark Part occurrence as End Item
        
        
        Additionally, this service operation validates the existing alignment and updates the alignment with the
        current DesignRevision information.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateDesignPartAlignment',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=DesignPartAlignmentResponse,
        )

    @classmethod
    def findRelatedDesignOrPartStructures(cls, input: List[ObjectAlignmentInput]) -> DesignPartAlignmentResponse:
        """
        This operation finds all the Design structures related to the input Part BOMLine or alternatively finds all the
        Part structures related to the input Design BOMLline  and returns a list of structure roots.  The input can be
        any BOMLine in the source Part/Design structure.
        """
        return cls.execute_soa_method(
            method_name='findRelatedDesignOrPartStructures',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=DesignPartAlignmentResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def linkPlantBOPtoBOE(cls, linkBOPtoBOEObjectInfo: List[LinkBOPtoBOEObjectInfo]) -> LinkPlantBOPtoBOEResponse:
        """
        This operation links  the input source i.e. Plant Bill of Process (Plant BOP) and target i.e. Bill of
        Equipement (BOE) lines. Linking creates GRM relation Mfg0MELinkedBOERel between MEAppearancePathNode associated
        with source and target lines.
        If either source or target line is already linked to any of the lines in the target or source structures
        respectively then the current link is removed and a new link is created. But this is not applicable for the top
        lines i.e, Mfg0MEPlantBOP and MEPlant.
        During link, objects are linked based on the preference MEBOPToBOEObjectTypeMap. The preference
        MEBOPToBOEObjectTypeMap holds the information about the mapping between the object types. 
        If a mapping exists between the given source and target type then the types are linked otherwise an error is
        returned in service data.
        """
        return cls.execute_soa_method(
            method_name='linkPlantBOPtoBOE',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='DataManagement',
            params={'linkBOPtoBOEObjectInfo': linkBOPtoBOEObjectInfo},
            response_cls=LinkPlantBOPtoBOEResponse,
        )

    @classmethod
    def synchronizePlantBOPAndBOE(cls, synchronizeBopAndBoeInputInfo: List[SynchronizeBOPAndBOEInputInfo]) -> SynchronizePlantBOPAndBOEResponse:
        """
        This operation synchronizes the input source and target lines. Plant Bill of Process (Plant BOP) lines and Bill
        of Equipement (BOE) lines acts as source or target lines. The decision regarding which lines will act as source
        or target line will be taken by the user. 
        Synchronizing the Plant BOP structure and BOE structure will result in 
        
        1. Creating missing elements in target. The type of the element to be created will be identified by the
        preference MEBOPToBOEObjectTypeMap. MEAppearancePathNode associated with the new elements created in target
        will be linked with the MEAppearancePathNode associated with the source line.
        
        2. Copying of attributes from source line to target line. The attributes that are to be synchronized will be
        identified by the preference MEBOPToBOEObjectTypeMap.
        
        3. If the option for remove obsolete twin is provided then the target lines which do not have
        Mfg0MELinkedBOERel will be removed from the target structure.
        
        Use cases:
        Use case 1: Sync objects from Twin Structure
        This operation synchronizes the linked Plant BOP lines, BOE lines and hierarchy under it by
        1. Copying the attributes values from source to target for already linked lines.
        
        2. Creating an object in target structure for each object in source structure that does not have any connected
        object. 
        
        3. Removing every object in target structure that does not have any connected object in the source structure. 
        (Only if user had asked for it through the removeObsoleteTwin option)
        
        4. If a station or station process is created, linked station will be consumed in the station process.
        """
        return cls.execute_soa_method(
            method_name='synchronizePlantBOPAndBOE',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='DataManagement',
            params={'synchronizeBopAndBoeInputInfo': synchronizeBopAndBoeInputInfo},
            response_cls=SynchronizePlantBOPAndBOEResponse,
        )

    @classmethod
    def createBOEfromPlantBOP(cls, rootBOPLines: List[BOMLine]) -> CreateBOEfromPlantBOPResponse:
        """
        For Manufacturing planning both Bill of Equipment (BOE) and Plant Bill of Process (Plant BOP) are required. BOE
        stations hold location information and contain the equipment required for manufacturing. Plant BOP stations are
        used to define the sequences of the processes and hold information about the parts to be assembled in the
        station. The validation of the manufacturing process usually requires the combination of BOE and Plant BOP.
        This operation will create the Plant structures for the given input Plant BOP structures. The decision to
        create objects will be based on a preference. The preference MEBOPToBOEObjectTypeMap will hold the information
        about the mapped objects types which needs to be created. The new objects created will have same attribute
        values as that of originating object.
        
        Use cases:
        Use case 1: User create Plant structure for the selected Plant Bill of Process (Plant BOP)
        New BOE structure will be ceated for given Plant BOP structure. As part of the structure creation,
        MEAppearancePathNode will also be created for each line in the source (Plant BOP) and target (Plant) structure.
        The MEAppearancePathNode object of Plant BOP line will be connected to MEAppearancePathNode created for object
        in Plant with a GRM Relation Mfg0MELinkedBOERel. Station (MEStation) created in Plant structure (MEPlant) for
        any Station Process (Mfg0MEProcStatn) in Plant BOP (Mfg0MEPlantBOP) will be consumed in Station Process
        (Mfg0MEProcStatn).
        """
        return cls.execute_soa_method(
            method_name='createBOEfromPlantBOP',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='DataManagement',
            params={'rootBOPLines': rootBOPLines},
            response_cls=CreateBOEfromPlantBOPResponse,
        )


class StructureSearchService(TcService):

    @classmethod
    def getBOEForPlantBOPScope(cls, searchInput: SearchBOEInputInfo) -> SearchBOEResponse:
        """
        This operation searches the input Plant BOPLines for the Bill of Equipment ( BOEs) based on the search criteria
        and returns a list of all the matched BOEs.
        """
        return cls.execute_soa_method(
            method_name='getBOEForPlantBOPScope',
            library='Internal-Manufacturing',
            service_date='2016_09',
            service_name='StructureSearch',
            params={'searchInput': searchInput},
            response_cls=SearchBOEResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, BOMWindow
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetGOPartSolutionsResponse(TcBaseObj):
    """
    The GetGOPartSolutionsResponse structure represents output of vector of APN of line of usages (part solution),
    instantiating architecture  bom window and the service data.
    
    :var louBomLine: vector of BOMLine of LOU
    :var archBomWindow: Instantiating Architecture Bom window
    :var serviceData: serviceData returned as response for retrieving
    information on LOU APN
    """
    louBomLine: List[BOMLine] = ()
    archBomWindow: BOMWindow = None
    serviceData: ServiceData = None


@dataclass
class GetPastePrimeInfo(TcBaseObj):
    """
    This structure contains the list of the source and target BOMLine objects and a flag to indicate which attributes
    needs to be copied from the source BOMLine to target BOMLine.
    
    :var parentBomLine: List of the target Architecture Breakdown or Architecture Breakdown Element BOMLine objects.
    
    :var componentBomLine: List of the source Architecture Breakdown Element BOMLine objects.
    
    :var flag: Flag to decide which attributes need to be copied from the source to the target BOMLine.
    If flag value is 1 then only variability will be copied from the source to target BOMLine.
    If flag value is 2 then variability and NVEs will be copied from the source to target BOMLine.
    If flag value is 3 then variability, NVEs and part solutions will be copied from the source to target BOMLine.
    """
    parentBomLine: List[BOMLine] = ()
    componentBomLine: List[BOMLine] = ()
    flag: int = 0
